package com.ym.mec.cms.controller;

import com.ym.mec.cms.dal.dao.StudentRegistrationDao;
import com.ym.mec.cms.dal.entity.SysConfig;
import com.ym.mec.cms.service.SysConfigService;
import com.ym.mec.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.cms.controller.queryinfo.NewsInformationQueryInfo;
import com.ym.mec.cms.dal.dao.SysNewsInformationDao;
import com.ym.mec.cms.dal.entity.SysNewsInformation;
import com.ym.mec.cms.service.SysNewsInformationService;
import com.ym.mec.cms.service.SysNewsTypeService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.web.WebFeignService;
import com.yonge.log.model.AuditLogAnnotation;

import javax.servlet.http.HttpServletRequest;

@RestController
@RequestMapping("news")
@Api(tags = "资讯服务")
public class NewsController extends BaseController {

	@Autowired
	private SysNewsInformationService sysNewsInformationService;
	
	@Autowired
	private SysNewsTypeService sysNewsTypeService;
	
	@Autowired
	private SysNewsInformationDao sysNewsInformationDao;
	
	@Autowired
	private SysUserFeignService sysUserFeignService;
	
	@Autowired
	private WebFeignService webFeignService;

	@Autowired
	private StudentRegistrationDao studentRegistrationDao;

	@Autowired
	private SysConfigService sysConfigService;

	private final LocalDate xiaMenBannerStartShowTime = LocalDate.parse("2021-10-10", DateUtil.dateFormatter);

	@ApiOperation("资讯列表分页查询")
	@GetMapping(value = "/list")
	public Object getList(NewsInformationQueryInfo queryInfo) {
		SysUser user = sysUserFeignService.queryUserInfo();

		String clientId = queryInfo.getClientId();
		
		if (!StringUtils.equalsIgnoreCase("student", clientId)) {
			queryInfo.setOrganIdList(webFeignService.queryOrganIdByUserId(user.getId(), clientId));
		} else {
			if (user == null || user.getId() == null) {
				queryInfo.setOrganId(43);
			} else {
				queryInfo.setOrganId(user.getOrganId());
			}
		}

		SysConfig cloudTeacherActiveIds = sysConfigService.findByParamName(SysConfigService.CLOUD_TEACHER_ACTIVE_BANNER_IDS);
		List<Integer> excludeIds = new ArrayList<>();
		if(Objects.nonNull(cloudTeacherActiveIds)&&StringUtils.isNotBlank(cloudTeacherActiveIds.getParanValue())){
			excludeIds = Arrays.stream(cloudTeacherActiveIds.getParanValue().split(",")).map(id->Integer.valueOf(id)).collect(Collectors.toList());
		}

		if (user == null || user.getId() == null) {
			queryInfo.setOrganId(43);
			queryInfo.setExcludeIds(excludeIds);
		} else {
			queryInfo.setOrganId(user.getOrganId());
			int memberGroupNum = studentRegistrationDao.countStudentMemberGroupNum(user.getId());

			//厦门学员
			boolean xiaMenNotShow = (user.getOrganId()==40 || user.getOrganId() == 23 || user.getOrganId() == 25)&&LocalDate.now().compareTo(xiaMenBannerStartShowTime)<0;
			if(memberGroupNum>0||xiaMenNotShow){
				queryInfo.setExcludeIds(excludeIds);
			}
		}
		
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);
		int count = sysNewsInformationDao.queryCount(params);
		if(count == 0){
			queryInfo.setMemo(null);
		}
		return succeed(sysNewsInformationService.queryPage(queryInfo));
	}

	@ApiOperation("资讯列表分页查询")
	@GetMapping(value = "/queryPage")
	public Object queryPage(NewsInformationQueryInfo queryInfo) {
		
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);
		int count = sysNewsInformationDao.queryCount(params);
		if(count == 0){
			queryInfo.setMemo(null);
		}
		return succeed(sysNewsInformationService.queryPage(queryInfo));
	}

	@ApiOperation("资讯类型列表查询")
	@GetMapping(value = "/typeList")
	public Object getTypeList(Integer parentId) {
		return succeed(sysNewsTypeService.queryByParentId(parentId));
	}

	@ApiOperation("资讯列表分页查询")
	@GetMapping(value = "/homeList")
	public Object getHomeList(NewsInformationQueryInfo queryInfo) {
		queryInfo.setRows(10);
		SysUser user = sysUserFeignService.queryUserInfo();
		SysConfig cloudTeacherActiveIds = sysConfigService.findByParamName(SysConfigService.CLOUD_TEACHER_ACTIVE_BANNER_IDS);
		List<Integer> excludeIds = new ArrayList<>();
		if(Objects.nonNull(cloudTeacherActiveIds)&&StringUtils.isNotBlank(cloudTeacherActiveIds.getParanValue())){
			excludeIds = Arrays.stream(cloudTeacherActiveIds.getParanValue().split(",")).map(id->Integer.valueOf(id)).collect(Collectors.toList());
		}
		if (user == null || user.getId() == null) {
			queryInfo.setOrganId(43);
			queryInfo.setExcludeIds(excludeIds);
		} else {
			queryInfo.setOrganId(user.getOrganId());
			int memberGroupNum = studentRegistrationDao.countStudentMemberGroupNum(user.getId());
			//厦门学员
			boolean xiaMenNotShow = (user.getOrganId()==40 || user.getOrganId() == 23 || user.getOrganId() == 25)&&LocalDate.now().compareTo(xiaMenBannerStartShowTime)<0;
			if(memberGroupNum>0||xiaMenNotShow){
				queryInfo.setExcludeIds(excludeIds);
			}
		}
		return succeed(sysNewsInformationService.getHomeList(user, queryInfo));
	}

	@ApiOperation("查询资讯详情")
	@ApiImplicitParam(name = "id", value = "资讯ID编号", required = true, dataType = "Long", paramType = "path")
	@GetMapping(value = "/query")
	public Object query(Long id) {

		return succeed(sysNewsInformationService.queryById(id));
	}

	@ApiOperation("新增资讯")
	@PostMapping(value = "/add", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@AuditLogAnnotation(operateName = "资讯新增",interfaceURL = "news/add")
	@PreAuthorize("@pcs.hasPermissions('news/add')")
	public Object add(SysNewsInformation newsInfo) {
		return succeed(sysNewsInformationService.insert(newsInfo));
	}

	@ApiOperation("更新资讯")
	@PostMapping(value = "/update", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@AuditLogAnnotation(operateName = "资讯更新",interfaceURL = "news/update")
	@PreAuthorize("@pcs.hasPermissions('news/update')")
	public Object update(SysNewsInformation newsInfo) {
		Date date = new Date();
		newsInfo.setUpdateTime(date);

		return succeed(sysNewsInformationService.update(newsInfo));
	}

	@ApiOperation("删除")
	@PostMapping(value = "/del/{id}", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@AuditLogAnnotation(operateName = "资讯删除",interfaceURL = "news/del")
	@PreAuthorize("@pcs.hasPermissions('news/del')")
	public Object add(@PathVariable("id") Long id) {
		return succeed(sysNewsInformationService.deleteWithLogical(id));
	}

}
