package com.ym.mec.auth.core.filter;

import java.io.IOException;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springframework.security.authentication.AbstractAuthenticationToken;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;

import com.ym.mec.auth.api.entity.LoginEntity;
import com.ym.mec.auth.config.token.PhoneAuthenticationToken;
import com.ym.mec.common.security.SecurityConstants;

public class PhoneLoginAuthenticationFilter extends AbstractAuthenticationProcessingFilter {

	private static final String SPRING_SECURITY_RESTFUL_PHONE_KEY = "phone";
	private static final String SPRING_SECURITY_RESTFUL_VERIFY_CODE_KEY = "smsCode";
	private static final String clientIdParameter = "clientId";
	private static final String IS_LESSEE = "isLessee";

	private static final String TENANT_ID = "tenantId";

	private static final String ORGAN_ID = "organId";
	
	private static final String DEVICE_NUM = "deviceNum";

	private static final String SPRING_SECURITY_RESTFUL_LOGIN_URL = "/smsLogin";
	private boolean postOnly = true;

	public PhoneLoginAuthenticationFilter() {
		super(new AntPathRequestMatcher(SPRING_SECURITY_RESTFUL_LOGIN_URL, "POST"));
	}

	@Override
	public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response) throws AuthenticationException {
		if (postOnly && !request.getMethod().equals("POST")) {
			throw new AuthenticationServiceException("Authentication method not supported: " + request.getMethod());
		}

		AbstractAuthenticationToken authRequest;
		// 手机验证码登陆
		String principal = obtainParameter(request, SPRING_SECURITY_RESTFUL_PHONE_KEY);
		String credentials = obtainParameter(request, SPRING_SECURITY_RESTFUL_VERIFY_CODE_KEY);
		// 是否是租户
		String isLessee = obtainParameter(request, IS_LESSEE);
		
		String tenantId = obtainParameter(request, TENANT_ID);
		
		String organId = obtainParameter(request, ORGAN_ID);

		boolean isRegister = StringUtils.equals("1", isLessee) || StringUtils.equals("true", isLessee);

		String clientId = request.getParameter(clientIdParameter).toUpperCase();

		if ("EDUCATION".equals(clientId)) {
			clientId = "SYSTEM";
		}
		
		String deviceNum = request.getParameter(DEVICE_NUM);

		principal = principal.trim();

		LoginEntity loginEntity = new LoginEntity();
		loginEntity.setClientId(clientId);
		loginEntity.setPhone(principal);
		loginEntity.setSmsCode(credentials);
		loginEntity.setIsRegister(isRegister);
		loginEntity.setDeviceNum(deviceNum);
		loginEntity.setOrganId(organId);
		loginEntity.setTenantId(tenantId);

		authRequest = new PhoneAuthenticationToken(SecurityConstants.PHONE_PRINCIPAL_PREFIX + principal, loginEntity);

		// Allow subclasses to set the "details" property
		setDetails(request, authRequest);
		return this.getAuthenticationManager().authenticate(authRequest);
	}

	@Override
	protected void successfulAuthentication(HttpServletRequest request, HttpServletResponse response, FilterChain chain, Authentication authResult)
			throws IOException, ServletException {
		super.successfulAuthentication(request, response, chain, authResult);
		// chain.doFilter(request, response);
	}

	private void setDetails(HttpServletRequest request, AbstractAuthenticationToken authRequest) {
		authRequest.setDetails(authenticationDetailsSource.buildDetails(request));
	}

	private String obtainParameter(HttpServletRequest request, String parameter) {
		String result = request.getParameter(parameter);
		return result == null ? "" : result;
	}

}
