package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.ActivityStudentDto;
import com.ym.mec.biz.dal.entity.ActivityUserMapper;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

@Component
public interface ActivityUserMapperDao extends BaseDAO<Integer, ActivityUserMapper> {


    /**
    * @description: 获取学员排课资格
    * @param activityId
    * @param studentIds
    * @param type 排课资格类型
    * @return com.ym.mec.biz.dal.entity.ActivityUserMapper
    * @author zx
    * @date 2021/10/11 16:31
    */
    List<ActivityUserMapper> findByStudentIdList(@Param("activityId") Integer activityId, @Param("studentIds") String studentIds, @Param("type") String type);

    /**
    * @description: 获取学员购买的活动次数
     * @param activityId
     * @param userId
    * @return int
    * @author zx
    * @date 2021/10/12 18:22
    */
    int countActivityBuyNum(@Param("activityId") Integer activityId, @Param("userId") Integer userId);

    /**
    * @description:
     * @param groupId
     * @param studentId
    * @return com.ym.mec.biz.dal.entity.ActivityUserMapper
    * @author zx
    * @date 2021/10/14 15:40
    */
    ActivityUserMapper findByStudentId(@Param("groupId") Long groupId, @Param("studentId") Integer studentId);

    /**
     * @description: 消耗排课资格
     * @param activityUserMapperIds
     * @param studentIds
     * @param type
     * @param groupId
     * @return int
     * @author zx
     * @date 2021/10/15 15:53
     */
    Integer use(@Param("activityUserMapperIds") String activityUserMapperIds,
            @Param("studentIds") String studentIds,
            @Param("type") String type,
            @Param("groupId") Long groupId);

    /**
    * @description: 批量新增
     * @param activityUserMappers
    * @return void
    * @author zx
    * @date 2021/10/20 13:38
    */
    void batchInsert(@Param("activityUserMappers") List<ActivityUserMapper> activityUserMappers);

    /**
     * @description:
     * @param groupId
 * @param groupType
    * @return com.ym.mec.biz.dal.entity.ActivityUserMapper
     */
    ActivityUserMapper findVipUserMapper(@Param("groupId") Long groupId,
                                         @Param("groupType") String groupType,
                                         @Param("studentId") Integer studentId);

    /**
     * @description: 修改指导老师
     * @param userId
     * @param teacherId
     * @return void
     * @author zx
     * @date 2021/11/1 16:21
     */
    void updateTeacherId(@Param("userId") Integer userId, @Param("teacherId") Integer teacherId);

    /**
    * @description: 获取学员活动排课资格列表
     * @param params
    * @return int
    * @author zx
    * @date 2021/11/15 16:38
    */
    int countActivityUserMapper(Map<String, Object> params);

    /**
     * @description: 获取学员活动排课资格列表
     * @param params
     * @return int
     * @author zx
     * @date 2021/11/15 16:38
     */
    List<ActivityStudentDto> queryActivityUserMapper(Map<String, Object> params);
}