package com.ym.mec.biz.service;

import com.ym.mec.biz.dal.dto.ActivityQueryDto;
import com.ym.mec.biz.dal.dto.ActivityStudentAdjustDto;
import com.ym.mec.biz.dal.dto.ActivityStudentDto;
import com.ym.mec.biz.dal.dto.ActivityUserMapperAddDto;
import com.ym.mec.biz.dal.entity.ActivityUserMapper;
import com.ym.mec.biz.dal.page.ActivityStudentQueryInfo;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.BaseService;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

public interface ActivityUserMapperService extends BaseService<Integer, ActivityUserMapper> {

    /**
    * @description:
     * @param activityId
     * @param studentIds
     * @param type
    * @return java.util.List<com.ym.mec.biz.dal.entity.ActivityUserMapper>
    * @author zx
    * @date 2021/10/15 15:48
    */
    List<ActivityUserMapper> findByStudentIdList(Integer activityId, String studentIds, String type);

    /**
    * @description: 消耗学员排课资格
     * @param activityUserMapperIds
     * @param studentIds
     * @param type
    * @return void
    * @author zx
    * @date 2021/10/15 15:51
    */
    int use(String activityUserMapperIds, String studentIds, String type,Long groupId);

    /**
    * @description:
     * @param groupId
     * @param studentId
    * @return com.ym.mec.biz.dal.entity.ActivityUserMapper
    * @author zx
    * @date 2021/10/15 16:16
    */
    ActivityUserMapper findByStudentId(Long groupId, Integer studentId);

    /**
    * @description: 获取活动购买次数
     * @param activityId
     * @param userId
    * @return int
    * @author zx
    * @date 2021/10/15 16:18
    */
    int countActivityBuyNum(Integer activityId, Integer userId);

    /**
    * @description: 批量新增
     * @param activityUserMappers
    * @return void
    * @author zx
    * @date 2021/10/20 13:37
    */
    void batchInsert(List<ActivityUserMapper> activityUserMappers);

    /**
    * @description:
     * @param groupId
     * @param groupType
    * @return com.ym.mec.biz.dal.entity.ActivityUserMapper
    * @author zx
    * @date 2021/10/22 17:35
    */
    ActivityUserMapper findVipUserMapper(Long groupId,String groupType,Integer studentId);

    /**
    * @description: 获取学员活动排课资格列表
     * @param queryInfo
    * @return java.lang.Object
    * @author zx
    * @date 2021/11/15 16:34
    */
    PageInfo<ActivityStudentDto> queryActivityUserMapper(ActivityStudentQueryInfo queryInfo);

    /**
    * @description: 添加排课资格
     * @param activityUserMapperAddDto
    * @return void
    * @author zx
    * @date 2021/11/18 11:19
    */
    void add(ActivityUserMapperAddDto activityUserMapperAddDto);

    /**
    * @description: 取消排课资格
     * @param activityUserMapperId
     * @param memo
    * @return void
    * @author zx
    * @date 2021/11/18 11:19
    */
    void cut(Integer activityUserMapperId, String memo, BigDecimal amount);

}