package com.ym.mec.biz.service.impl;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.ActivityQueryDto;
import com.ym.mec.biz.dal.dto.ActivityStudentAdjustDto;
import com.ym.mec.biz.dal.dto.ActivityStudentDto;
import com.ym.mec.biz.dal.dto.ActivityUserMapperAddDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.ReturnFeeEnum;
import com.ym.mec.biz.dal.page.ActivityStudentQueryInfo;
import com.ym.mec.biz.service.ActivityUserMapperService;
import com.ym.mec.common.constant.CommonConstants;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.enums.GroupType.ACTIVITY;
import static com.ym.mec.biz.dal.enums.GroupType.SUBJECT_CHANGE;

@Service
public class ActivityUserMapperServiceImpl extends BaseServiceImpl<Integer, ActivityUserMapper>  implements ActivityUserMapperService {
	
	@Autowired
	private ActivityUserMapperDao activityUserMapperDao;
	@Autowired
	private ActivityUserMapperAdjustLogDao activityUserMapperAdjustLogDao;
	@Autowired
	private VipGroupActivityDao vipGroupActivityDao;
	@Autowired
	private StudentDao studentDao;
	@Autowired
	private TeacherDao teacherDao;
	@Autowired
	private SysUserCashAccountLogDao sysUserCashAccountLogDao;
	@Autowired
	private SysUserFeignService sysUserFeignService;

	@Override
	public BaseDAO<Integer, ActivityUserMapper> getDAO() {
		return activityUserMapperDao;
	}

    @Override
    public List<ActivityUserMapper> findByStudentIdList(Integer activityId, String studentIds, String type) {
        return activityUserMapperDao.findByStudentIdList(activityId,studentIds,type);
    }

	@Override
	public int use(String activityUserMapperIds, String studentIds, String type,Long groupId) {
		return activityUserMapperDao.use(activityUserMapperIds,studentIds,type,groupId);
	}

	@Override
	public ActivityUserMapper findByStudentId(Long groupId, Integer studentId) {
		return activityUserMapperDao.findByStudentId(groupId,studentId);
	}

	@Override
	public int countActivityBuyNum(Integer activityId, Integer userId) {
		return activityUserMapperDao.countActivityBuyNum(activityId,userId);
	}

    @Override
    public void batchInsert(List<ActivityUserMapper> activityUserMappers) {
		activityUserMapperDao.batchInsert(activityUserMappers);
    }

	@Override
	public ActivityUserMapper findVipUserMapper(Long groupId, String groupType,Integer studentId) {
		return activityUserMapperDao.findVipUserMapper(groupId,groupType,studentId);
	}

    @Override
    public PageInfo<ActivityStudentDto> queryActivityUserMapper(ActivityStudentQueryInfo queryInfo) {
		PageInfo<ActivityStudentDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<ActivityStudentDto> dataList = null;
		int count = activityUserMapperDao.countActivityUserMapper(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = activityUserMapperDao.queryActivityUserMapper(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
    }

    @Override
	@Transactional(rollbackFor = Exception.class)
    public void add(ActivityUserMapperAddDto activityUserMapperAddDto) {
		List<ActivityStudentAdjustDto> activityStudentAdjustDtos = activityUserMapperAddDto.getActivityStudentAdjustDtos();
		if(activityStudentAdjustDtos == null || activityStudentAdjustDtos.size() == 0){
        	throw new BizException("请选择学员");
		}
		Integer activityId = activityUserMapperAddDto.getActivityId();
		String memo = activityUserMapperAddDto.getMemo();
		VipGroupActivity activity = vipGroupActivityDao.get(activityId);
		if(activity == null){
			throw new BizException("活动信息不存在");
		}
		if(activity.getActivityType() != 0){
			throw new BizException("操作失败:只有课程活动支持此操作");
		}
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		Integer operatorId = sysUser.getId();
		String operatorName = sysUser.getRealName();

		List<Integer> studentIds = activityStudentAdjustDtos.stream().map(e -> e.getUserId()).collect(Collectors.toList());
		List<Student> studentTeacher = studentDao.findByStudentIds(studentIds);
		Map<Integer, List<Student>> collect = studentTeacher.stream().collect(Collectors.groupingBy(Student::getUserId));
		String courseType = activity.getCourseType();
		Integer giveCourseNum = activity.getGiveCourseNum();
		String giveCourseType = activity.getGiveCourseType();
		List<ActivityUserMapper> activityUserMappers = new ArrayList<>();
		List<ActivityUserMapperAdjustLog> activityUserMapperAdjustLogs = new ArrayList<>();
		for (ActivityStudentAdjustDto adjustDto : activityStudentAdjustDtos) {
			//添加操作记录
			ActivityUserMapperAdjustLog activityUserMapperAdjustLog = new ActivityUserMapperAdjustLog();
			activityUserMapperAdjustLog.setActivityId(activityId);
			activityUserMapperAdjustLog.setOperatorId(operatorId);
			activityUserMapperAdjustLog.setOperatorName(operatorName);
			activityUserMapperAdjustLog.setCourseNum(adjustDto.getCourseNum());
			activityUserMapperAdjustLog.setGiveCourseNum(adjustDto.getGiveCourseNum());
			activityUserMapperAdjustLog.setMemo(memo);
			activityUserMapperAdjustLog.setUserId(adjustDto.getUserId());
			activityUserMapperAdjustLog.setType(1);
			activityUserMapperAdjustLogs.add(activityUserMapperAdjustLog);

			int activityUserMapperNum = adjustDto.getCourseNum();
			if(giveCourseNum != null && giveCourseNum > 0){
				if(adjustDto.getGiveCourseNum() > activityUserMapperNum){
					activityUserMapperNum = adjustDto.getGiveCourseNum();
				}
			}
			if(activityUserMapperNum <= 0){
				throw new BizException("操作失败:加课次数不可为空");
			}
			List<Student> students = collect.get(adjustDto.getUserId());
			Integer teacherId = null;
			if(students != null && students.size() > 0){
				teacherId = students.get(0).getTeacherId();
			}
			for (int i = 0; i < activityUserMapperNum; i++) {
				ActivityUserMapper activityUserMapper = new ActivityUserMapper();
				activityUserMapper.setActivityId(activityId);
				activityUserMapper.setUserId(adjustDto.getUserId());
				activityUserMapper.setActualPrice(BigDecimal.ZERO);
				activityUserMapper.setReturnFee(false);
				activityUserMapper.setTeacherId(teacherId);
				activityUserMapper.setAddMemo(memo);
				if(adjustDto.getCourseNum() > i){
					if("VIP".equals(courseType)){
						activityUserMapper.setVipFlag(1);
					}else {
						activityUserMapper.setPracticeFlag(1);
					}
				}

				if(adjustDto.getGiveCourseNum() > i && giveCourseNum != null && giveCourseNum > 0){
					if("VIP".equals(giveCourseType)){
						activityUserMapper.setGiveVipFlag(1);
					}else {
						activityUserMapper.setGivePracticeFlag(1);
					}
				}
				activityUserMappers.add(activityUserMapper);
			}
		}
		activityUserMapperAdjustLogDao.batchInsert(activityUserMapperAdjustLogs);
		activityUserMapperDao.batchInsert(activityUserMappers);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void cut(Integer activityUserMapperId, String memo, BigDecimal amount) {
		ActivityUserMapper activityUserMapper = activityUserMapperDao.get(activityUserMapperId);
		if(activityUserMapper.getVipFlag() == 2 ||
				activityUserMapper.getGiveVipFlag() == 2 ||
				activityUserMapper.getPracticeFlag() == 2 ||
				activityUserMapper.getGivePracticeFlag() == 2 || activityUserMapper.getReturnFee()){
			throw new BizException("操作失败：活动已消耗");
		}
		activityUserMapper.setCutMemo(memo);
		activityUserMapper.setReturnFee(true);
		activityUserMapperDao.update(activityUserMapper);

		SysUser sysUser = sysUserFeignService.queryUserInfo();
		Integer operatorId = sysUser.getId();
		String operatorName = sysUser.getRealName();
		//添加操作记录
		ActivityUserMapperAdjustLog activityUserMapperAdjustLog = new ActivityUserMapperAdjustLog();
		activityUserMapperAdjustLog.setActivityId(activityUserMapper.getActivityId());
		activityUserMapperAdjustLog.setOperatorId(operatorId);
		activityUserMapperAdjustLog.setOperatorName(operatorName);
		if(activityUserMapper.getVipFlag() == 1 || activityUserMapper.getPracticeFlag() == 1){
			activityUserMapperAdjustLog.setCourseNum(-1);
		}
		if(activityUserMapper.getGiveVipFlag() == 1 || activityUserMapper.getGivePracticeFlag() == 1){
			activityUserMapperAdjustLog.setGiveCourseNum(-1);
		}
		activityUserMapperAdjustLog.setMemo(memo);
		activityUserMapperAdjustLog.setUserId(activityUserMapper.getUserId());
		activityUserMapperAdjustLog.setType(0);
		activityUserMapperAdjustLog.setActivityUserMapperId(activityUserMapperId);
		activityUserMapperAdjustLogDao.insert(activityUserMapperAdjustLog);
		if(amount == null || amount.compareTo(BigDecimal.ZERO) <= 0){
			return;
		}
		BigDecimal actualPrice = activityUserMapper.getActualPrice().multiply(new BigDecimal(0.995)).setScale(CommonConstants.DECIMAL_FINAL_PLACE, BigDecimal.ROUND_HALF_UP);
		if(amount.compareTo(actualPrice) > 0){
			throw new BizException("操作失败：退费金额不得超过购买金额{}",actualPrice);
		}
		SysUser user = teacherDao.getUser(activityUserMapper.getUserId());
		SysUserCashAccountLog sysUserCashAccountLog = new SysUserCashAccountLog();
		sysUserCashAccountLog.setUserId(activityUserMapper.getUserId());
		sysUserCashAccountLog.setGroupType(ACTIVITY);
		sysUserCashAccountLog.setOrganId(user.getOrganId());
		sysUserCashAccountLog.setGroupId(activityUserMapperId.toString());
		sysUserCashAccountLog.setAmount(amount);
		sysUserCashAccountLog.setReturnFeeType(ReturnFeeEnum.CANCEL_ACTIVITY_COURSE);
		sysUserCashAccountLog.setComment("取消活动排课");
		sysUserCashAccountLogDao.insert(sysUserCashAccountLog);
	}
}