package com.ym.mec.biz.service.impl;


import com.alibaba.druid.sql.visitor.functions.If;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.entity.SysUserDevice;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.dao.FinancialExpenditureDao;
import com.ym.mec.biz.dal.dao.OrganizationDao;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.entity.FinancialExpenditure;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.FinancialExpenditureQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.excel.POIUtil;
import com.ym.mec.util.ini.IniFileUtil;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class FinancialExpenditureServiceImpl extends BaseServiceImpl<Long, FinancialExpenditure> implements FinancialExpenditureService {

    @Autowired
    private FinancialExpenditureDao financialExpenditureDao;
    @Autowired
    private OrganizationDao organizationDao;
    @Autowired
    private EmployeeDao employeeDao;
    @Autowired
    private VipGroupService vipGroupService;
    @Autowired
    private GroupClassService groupClassService;
    @Autowired
    private MusicGroupService musicGroupService;

    @Override
    public BaseDAO<Long, FinancialExpenditure> getDAO() {
        return financialExpenditureDao;
    }

    private static final Logger LOGGER = LoggerFactory.getLogger(FinancialExpenditureServiceImpl.class);

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<FinancialExpenditure> importFinancialExpenditure(MultipartFile file) throws Exception {
        Map<String, List<Map<String, Object>>> sheetsListMap = POIUtil.importExcel(new ByteArrayInputStream(file.getBytes()), 2, file.getOriginalFilename());
        InputStream inputStream = new ClassPathResource("columnMapper.ini").getInputStream();
        Map<String, String> columns = IniFileUtil.readIniFile(inputStream, TemplateTypeEnum.FINANCIAL_EXPENDITURE.getMsg());
        List<FinancialExpenditure> financialExpenditures = new ArrayList<>();
        Map<String, Integer> organMap = getMap("organization", "name_", "id_", true, String.class, Integer.class);
        Map<String, Integer> cooperationOrganMap = getMap("cooperation_organ", "name_", "id_", true, String.class, Integer.class);
        StringBuffer sb = new StringBuffer();
        for (String e : sheetsListMap.keySet()) {
            List<Map<String, Object>> sheet = sheetsListMap.get(e);
            List<Object> collect = sheet.stream().map(m -> m.get("钉钉流程编号")).collect(Collectors.toList());
            if (collect == null || collect.size() == 0) {
                continue;
            }
            valueIsNull:
            for (int j = 0; j < sheet.size(); j++) {
                int rowNum = j + 2;
                Map<String, Object> row = sheet.get(j);
                if (row.size() == 0) {
                    continue;
                }
                JSONObject objectMap = new JSONObject();
                for (String s : row.keySet()) {
                    if (!columns.containsKey(s)) {
                        continue;
                    }
                    String columnValue = columns.get(s);
                    if (columnValue.equals("type")) {
                        for (ExpenditureTypeEnum expenditureType : ExpenditureTypeEnum.values()) {
                            if (expenditureType.getDesc().equals(row.get(s).toString())) {
                                objectMap.put(columnValue, expenditureType);
                                break;
                            }
                        }
                        continue;
                    }
                    if (columnValue.equals("dingtalkProcessNo")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第" + rowNum).append("行数据导入失败：钉钉流程编号不可为空;");
                            continue valueIsNull;
                        } else {
                            objectMap.put("incomeType", row.get(s));
                        }
                    }
                    if (columnValue.equals("feeProject")) {
                        for (FeeProjectEnum feeProject : FeeProjectEnum.values()) {
                            if (feeProject.getDesc().equals(row.get(s).toString())) {
                                objectMap.put(columnValue, feeProject);
                                break;
                            }
                        }
                        continue;
                    }
                    if (columnValue.equals("amount")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第" + rowNum).append("行数据导入失败：费用不可为空;");
                            continue valueIsNull;
                        } else {
                            objectMap.put("amount", row.get(s));
                        }
                        continue;
                    }
                    if (columnValue.equals("organName") && StringUtils.isNotEmpty(row.get(s).toString())) {
                        Integer integer = organMap.get(row.get(s));
                        if (integer == null) {
                            sb.append("第" + rowNum).append("行数据导入失败：分部校验失败;");
                            continue valueIsNull;
                        } else {
                            objectMap.put("organId", integer);
                        }
                        continue;
                    }
                    if (columnValue.equals("cooperationOrganName") && StringUtils.isNotEmpty(row.get(s).toString())) {
                        Integer integer = cooperationOrganMap.get(row.get(s));
                        if (integer == null) {
                            sb.append("第" + rowNum).append("行数据导入失败：合作单位校验失败;");
                            continue valueIsNull;
                        } else {
                            objectMap.put("cooperationOrganId", integer);
                        }
                        continue;
                    }
                    if (columnValue.equals("paymentTime") && StringUtils.isNotEmpty(row.get(s).toString())) {
                        String toString = row.get(s).toString();
                        if (StringUtils.isNotEmpty(toString)) {
                            Date date = DateUtil.stringToDate(toString, DateUtil.DEFAULT_PATTERN);
                            if (date == null) {
                                throw new BizException("导入数据错误  付款时间格式错误");
                            } else {
                                objectMap.put(columnValue, DateUtil.format(date, DateUtil.DEFAULT_PATTERN));
                            }
                        }
                        continue;
                    }
                    objectMap.put(columnValue, row.get(s));
                }
                FinancialExpenditure financialExpenditure = null;
                try {
                    financialExpenditure = JSONObject.parseObject(objectMap.toJSONString(), FinancialExpenditure.class);
                    financialExpenditures.add(financialExpenditure);
                } catch (Exception ex) {
                    throw new BizException("导入数据出错,请检查Excel表格");
                }
            }
        }
        if (sb.length() > 0) {
            throw new BizException(sb.toString());
        }
        if (financialExpenditures.size() != 0) {
            financialExpenditureDao.batchInsert(financialExpenditures);
        }
        return financialExpenditures;
    }

    @Override
    public PageInfo<FinancialExpenditureDto> queryFinancialExpenditurePage(FinancialExpenditureQueryInfo queryInfo) {
        PageInfo<FinancialExpenditureDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<FinancialExpenditureDto> dataList = null;
        int count = financialExpenditureDao.findFinancialExpenditureCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = financialExpenditureDao.queryFinancialExpenditurePage(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public void batchDel(String ids) {
        if (StringUtils.isEmpty(ids)) {
            throw new BizException("参数校验失败");
        }
        financialExpenditureDao.batchDel(ids);
    }

    @Override
    public List<FinancialExpenditure> batchAdd(List<FinancialExpenditure> financialExpenditures) {
        financialExpenditureDao.batchInsert(financialExpenditures);
        return financialExpenditures;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void syncOaPayLog(Integer workOrderId) {
        PWorkOrderInfo pWorkOrderInfo = financialExpenditureDao.getWorkOrderInfo(workOrderId);
        if (pWorkOrderInfo != null){
            Integer hasFinancial = financialExpenditureDao.findByBatchNoAndProcessNo(workOrderId);
            if(hasFinancial != null){
                return;
            }
            String formStructure = financialExpenditureDao.getFormStructure(workOrderId);
            JSONObject formData = JSONObject.parseObject(financialExpenditureDao.getFormData(workOrderId));
            OaFormStructureDto oaFormStructureDto = JSONObject.parseObject(formStructure, OaFormStructureDto.class);
            List<OaInputDto> oaInputDtos = oaFormStructureDto.getList();
            //课程退费和乐团退费需要单独处理
            if(pWorkOrderInfo.getProcess().equals(22) || pWorkOrderInfo.getProcess().equals(19)){
                List<FinancialExpenditure> financialExpenditureList = new ArrayList<>();
                for (OaInputDto oaInputDto : oaInputDtos) {
                    if(!"子表单".equals(oaInputDto.getName())){
                        continue;
                    }
                    String submitForm = oaInputDto.getModel();
                    List<HashMap> hashMaps = JSONObject.parseArray(formData.get(submitForm).toString(), HashMap.class);
                    for (HashMap hashMap : hashMaps) {
                        FinancialExpenditure financialExpenditure = new FinancialExpenditure();
                        financialExpenditure.setBatchNo(pWorkOrderInfo.getId().toString());
                        financialExpenditure.setFinancialProcessNo(pWorkOrderInfo.getId().toString());
                        List<OaColumnDto> columns = oaInputDto.getColumns();
                        if(columns != null && columns.size() > 0){
                            for (OaColumnDto column : columns) {
                                List<OaInputDto> columnList = column.getList();
                                if(columnList != null && columnList.size() > 0){

                                    Employee employee = employeeDao.get(pWorkOrderInfo.getCreator());
                                    List<SimpleUserDto> byIds = new ArrayList<>();
                                    if(employee != null){
                                        List<Integer> userIds = new ArrayList<>();
                                        userIds.add(employee.getUserId());
                                        byIds = employeeDao.findByIds(userIds);
                                    }
                                    if(employee != null){
                                        if(byIds != null && byIds.size() > 0){
                                            SimpleUserDto simpleUserDto = byIds.get(0);
                                            financialExpenditure.setApplyUser(simpleUserDto.getUserName());
                                        }
                                    }
                                    for (OaInputDto inputDto : columnList) {
                                        String name = inputDto.getName();
                                        if(name.contains("课程组编号")){
                                            Object o = hashMap.get(inputDto.getModel());
                                            if(o != null){
                                                financialExpenditure.setVipGroupId(Long.parseLong(o.toString()));
                                                continue;
                                            }
                                        }else if(name.contains("课程类型")){
                                            Object o = hashMap.get(inputDto.getModel());
                                            if(o != null){
                                                financialExpenditure.setGroupType(o.toString());
                                                continue;
                                            }
                                        }else if(name.contains("乐团编号")){
                                            Object o = hashMap.get(inputDto.getModel());
                                            if(o != null){
                                                financialExpenditure.setMusicGroupId(o.toString());
                                                continue;
                                            }
                                        }else if(name.contains("退费项目")){
                                            Object o = hashMap.get(inputDto.getModel());
                                            if(o != null){
                                                financialExpenditure.setReturnFeeType(o.toString());
                                                continue;
                                            }
                                        }else if(name.contains("学员编号")){
                                            Object o = hashMap.get(inputDto.getModel());
                                            if(o != null){
                                                financialExpenditure.setStudentId(Integer.parseInt(o.toString()));
                                                continue;
                                            }
                                        }else if(name.contains("分部")){
                                            Object o = hashMap.get(inputDto.getModel());
                                            if(o != null){
                                                int organId = Integer.parseInt(o.toString());
                                                Organization organization = organizationDao.get(organId);
                                                if(organization != null){
                                                    financialExpenditure.setOrganId(organization.getId());
                                                    financialExpenditure.setOrganName(organization.getName());
                                                }
                                                continue;
                                            }
                                        }else if(name.contains("金额")){
                                            Object o = hashMap.get(inputDto.getModel());
                                            if(o != null){
                                                financialExpenditure.setAmount(new BigDecimal(Double.parseDouble(o.toString())));
                                                continue;
                                            }
                                        }else if(name.contains("情况说明")){
                                            Object o = hashMap.get(inputDto.getModel());
                                            if(o != null){
                                                financialExpenditure.setCause(o.toString());
                                                continue;
                                            }
                                        }else if(name.contains("支出类型")){
                                            Object o = hashMap.get(inputDto.getModel());
                                            if(o != null){
                                                financialExpenditure.setType(ExpenditureTypeEnum.valueOfDesc(o.toString()));
                                                continue;
                                            }
                                        }else if(name.contains("费用类型")){
                                            Object o = hashMap.get(inputDto.getModel());
                                            if(o != null){
                                                financialExpenditure.setFeeProject(FeeProjectEnum.valueOfDesc(o.toString()));
                                                continue;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        financialExpenditureList.add(financialExpenditure);
                    }
                }
                //课程退费
                if(pWorkOrderInfo.getProcess().equals(22)){
                    for (FinancialExpenditure financialExpenditure : financialExpenditureList) {
                        ReturnFeeDto returnFeeDto = new ReturnFeeDto();
                        returnFeeDto.setGroupType(GroupType.valueOfDesc(financialExpenditure.getGroupType()));
                        returnFeeDto.setAmount(financialExpenditure.getAmount());
                        returnFeeDto.setStudentId(financialExpenditure.getStudentId());
                        returnFeeDto.setConfirmReturnActivityGive(true);
                        returnFeeDto.setGroupId(financialExpenditure.getVipGroupId());
                        returnFeeDto.setVipGroupId(financialExpenditure.getVipGroupId());
                        if(returnFeeDto.getGroupType() == GroupType.VIP){
                            vipGroupService.applyRefundForStudentOa(returnFeeDto);
                        }else if(returnFeeDto.getGroupType() == GroupType.COMM || returnFeeDto.getGroupType() == GroupType.PRACTICE){
                            groupClassService.cancelGroupOa(returnFeeDto);
                        }
                    }
                }else {
                    //退团
                    for (FinancialExpenditure financialExpenditure : financialExpenditureList) {
                        Boolean isRefundInstrumentFee = false;
                        Boolean isRefundTeachingAssistantsFee = false;
                        Boolean maintenanceFee = false;
                        String returnFeeType = financialExpenditure.getReturnFeeType();
                        if(StringUtils.isNotEmpty(returnFeeType)){
                            if(returnFeeType.contains("乐器")){
                                isRefundInstrumentFee = true;
                            }
                            if(returnFeeType.contains("乐保")){
                                maintenanceFee = true;
                            }
                            if(returnFeeType.contains("教辅")){
                                isRefundTeachingAssistantsFee = true;
                            }
                        }
                        musicGroupService.directQuitMusicGroupOa(financialExpenditure.getMusicGroupId(), financialExpenditure.getStudentId(),
                                financialExpenditure.getCause(), isRefundInstrumentFee,
                                isRefundTeachingAssistantsFee, maintenanceFee, financialExpenditure.getAmount());
                    }
                }
                financialExpenditureDao.batchInsert(financialExpenditureList);
            }else {
                FinancialExpenditure financialExpenditure = new FinancialExpenditure();
                financialExpenditure.setBatchNo(workOrderId.toString());
                financialExpenditure.setFinancialProcessNo(workOrderId.toString());
                Employee employee = employeeDao.get(pWorkOrderInfo.getCreator());
                Integer organId = financialExpenditureDao.getDeptId(pWorkOrderInfo.getDeptId());
                Organization organization = organizationDao.get(organId);
                if(organization != null){
                    financialExpenditure.setOrganId(organization.getId());
                    financialExpenditure.setOrganName(organization.getName());
                }
                if(employee != null){
                    List<Integer> userIds = new ArrayList<>();
                    userIds.add(employee.getUserId());
                    List<SimpleUserDto> byIds = employeeDao.findByIds(userIds);
                    if(byIds != null && byIds.size() > 0){
                        SimpleUserDto simpleUserDto = byIds.get(0);
                        financialExpenditure.setApplyUser(simpleUserDto.getUserName());
                    }
                }
                for (OaInputDto oaInputDto : oaInputDtos) {
                    String name = oaInputDto.getName();
                    if(name.contains("金额")){
                        Object o = formData.get(oaInputDto.getModel());
                        if(o != null){
                            financialExpenditure.setAmount(new BigDecimal(Double.parseDouble(o.toString())));
                            continue;
                        }
                    }else if(name.contains("情况说明")){
                        Object o = formData.get(oaInputDto.getModel());
                        if(o != null){
                            financialExpenditure.setCause(o.toString());
                            continue;
                        }
                    }else if(name.contains("支出类型")){
                        Object o = formData.get(oaInputDto.getModel());
                        if(o != null){
                            financialExpenditure.setType(ExpenditureTypeEnum.valueOfDesc(o.toString()));
                            continue;
                        }
                    }else if(name.contains("费用类型")){
                        Object o = formData.get(oaInputDto.getModel());
                        if(o != null){
                            financialExpenditure.setFeeProject(FeeProjectEnum.valueOfDesc(o.toString()));
                            continue;
                        }
                    }
                }
                financialExpenditureDao.insert(financialExpenditure);
            }
        }
    }

    @Override
    public void checkCourseReturnFee(Map<String,String> paramMap) {
        String tplInfoId = paramMap.get("tplInfoId");
        if(StringUtils.isEmpty(tplInfoId)){
            throw new BizException("参数校验失败");
        }
        if(tplInfoId.equals("11") || tplInfoId.equals("8")){
            JSONObject formData = JSONObject.parseObject(paramMap.get("formData"));
            String formStructure = financialExpenditureDao.getTplInfo(tplInfoId);
            OaFormStructureDto oaFormStructureDto = JSONObject.parseObject(formStructure, OaFormStructureDto.class);
            List<OaInputDto> oaInputDtos = oaFormStructureDto.getList();
            List<FinancialExpenditure> financialExpenditureList = new ArrayList<>();
            for (OaInputDto oaInputDto : oaInputDtos) {
                if(!"子表单".equals(oaInputDto.getName())){
                    continue;
                }
                String submitForm = oaInputDto.getModel();
                List<HashMap> hashMaps = JSONObject.parseArray(formData.get(submitForm).toString(), HashMap.class);
                for (HashMap hashMap : hashMaps) {
                    FinancialExpenditure financialExpenditure = new FinancialExpenditure();
                    List<OaColumnDto> columns = oaInputDto.getColumns();
                    if(columns != null && columns.size() > 0){
                        for (OaColumnDto column : columns) {
                            List<OaInputDto> columnList = column.getList();
                            if(columnList != null && columnList.size() > 0){
                                for (OaInputDto inputDto : columnList) {
                                    String name = inputDto.getName();
                                    if(name.contains("课程组编号")){
                                        Object o = hashMap.get(inputDto.getModel());
                                        if(o != null){
                                            financialExpenditure.setVipGroupId(Long.parseLong(o.toString()));
                                            continue;
                                        }
                                    }else if(name.contains("课程类型")){
                                        Object o = hashMap.get(inputDto.getModel());
                                        if(o != null){
                                            financialExpenditure.setGroupType(o.toString());
                                            continue;
                                        }
                                    }else if(name.contains("乐团编号")){
                                        Object o = hashMap.get(inputDto.getModel());
                                        if(o != null){
                                            financialExpenditure.setMusicGroupId(o.toString());
                                            continue;
                                        }
                                    }else if(name.contains("学员编号")){
                                        Object o = hashMap.get(inputDto.getModel());
                                        if(o != null){
                                            financialExpenditure.setStudentId(Integer.parseInt(o.toString()));
                                            continue;
                                        }
                                    }else if(name.contains("金额")){
                                        Object o = hashMap.get(inputDto.getModel());
                                        if(o != null){
                                            financialExpenditure.setAmount(new BigDecimal(Double.parseDouble(o.toString())));
                                            continue;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    financialExpenditureList.add(financialExpenditure);
                }
            }
            //课程退费
            if(tplInfoId.equals("11")){
                for (FinancialExpenditure financialExpenditure : financialExpenditureList) {
                    ReturnFeeDto returnFeeDto = new ReturnFeeDto();
                    returnFeeDto.setGroupType(GroupType.valueOfDesc(financialExpenditure.getGroupType()));
                    returnFeeDto.setAmount(financialExpenditure.getAmount());
                    returnFeeDto.setStudentId(financialExpenditure.getStudentId());
                    returnFeeDto.setConfirmReturnActivityGive(true);
                    returnFeeDto.setGroupId(financialExpenditure.getVipGroupId());
                    returnFeeDto.setVipGroupId(financialExpenditure.getVipGroupId());
                    if(returnFeeDto.getGroupType() == GroupType.VIP){
                        vipGroupService.checkApplyRefundForStudentOa(returnFeeDto);
                    }else if(returnFeeDto.getGroupType() == GroupType.COMM || returnFeeDto.getGroupType() == GroupType.PRACTICE){
                        groupClassService.checkCancelGroupOa(returnFeeDto);
                    }
                }
            }else {
                for (FinancialExpenditure financialExpenditure : financialExpenditureList) {
                    musicGroupService.checkDirectQuitMusicGroupOa(financialExpenditure.getMusicGroupId(), financialExpenditure.getStudentId());
                }
            }
        }
    }
}
