package com.ym.mec.biz.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.ym.mec.biz.dal.dto.MemberPayParamDto;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.entity.HttpResponseResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.MemberRankPrivilegesDao;
import com.ym.mec.biz.dal.dao.MemberRankSettingDao;
import com.ym.mec.biz.dal.entity.CloudTeacherOrder;
import com.ym.mec.biz.dal.entity.MemberFeeSetting;
import com.ym.mec.biz.dal.entity.MemberRankPrivileges;
import com.ym.mec.biz.dal.entity.MemberRankSetting;
import com.ym.mec.biz.dal.entity.Student;
import com.ym.mec.biz.dal.entity.StudentPaymentOrder;
import com.ym.mec.biz.dal.entity.SysUserCashAccount;
import com.ym.mec.biz.dal.entity.SysUserCashAccountDetail;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.date.DateUtil;

@Service
public class MemberRankSettingServiceImpl extends BaseServiceImpl<Integer, MemberRankSetting> implements MemberRankSettingService {

	@Autowired
	private MemberRankSettingDao memberRankSettingDao;
	@Autowired
	private MemberRankPrivilegesDao memberRankPrivilegesDao;

	@Autowired
	private MemberFeeSettingService memberFeeSettingService;

	@Autowired
	private StudentService studentService;

	@Autowired
	private StudentPaymentOrderService studentPaymentOrderService;

	@Autowired
	private SysUserCashAccountService sysUserCashAccountService;

	@Autowired
	private SysUserCashAccountDetailService sysUserCashAccountDetailService;

	@Autowired
	private ContractService contractService;

	@Autowired
	private SysUserFeignService sysUserFeignService;

	@Autowired
	private CloudTeacherOrderService cloudTeacherOrderService;

	@Autowired
	private IdGeneratorService idGeneratorService;

	@Autowired
	private SysCouponCodeService sysCouponCodeService;

	private final Logger logger = LoggerFactory.getLogger(this.getClass());

	@Override
	public BaseDAO<Integer, MemberRankSetting> getDAO() {
		return memberRankSettingDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void add(MemberRankSetting memberRankSetting) {
		memberRankSettingDao.insert(memberRankSetting);
		List<Integer> memberPrivilegesItemIdList = memberRankSetting.getMemberPrivilegesItemIdList();
		if (memberPrivilegesItemIdList == null || memberPrivilegesItemIdList.size() == 0) {
			throw new BizException("操作失败：请配置会员权益");
		}
		memberRankPrivilegesDao.batchInsert(memberRankSetting.getId(), memberPrivilegesItemIdList);
	}

	@Override
	public MemberRankSetting getDetail(Integer memberRankSettingId) {
		MemberRankSetting memberRankSetting = memberRankSettingDao.get(memberRankSettingId);
		List<MemberRankPrivileges> byRankSettingId = memberRankPrivilegesDao.findByRankSettingId(memberRankSettingId);
		if (byRankSettingId != null && byRankSettingId.size() > 0) {
			memberRankSetting.setMemberRankPrivileges(byRankSettingId);
			memberRankSetting.setMemberPrivilegesItemIdList(byRankSettingId.stream().map(e -> e.getMemberPrivilegesId()).collect(Collectors.toList()));
		}
		return memberRankSetting;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateSetting(MemberRankSetting memberRankSetting) {
		memberRankSettingDao.update(memberRankSetting);
		memberRankPrivilegesDao.deleteByRankSettingId(memberRankSetting.getId());
		List<Integer> memberPrivilegesItemIdList = memberRankSetting.getMemberPrivilegesItemIdList();
		if (memberPrivilegesItemIdList != null && memberPrivilegesItemIdList.size() > 0) {
			memberRankPrivilegesDao.batchInsert(memberRankSetting.getId(), memberPrivilegesItemIdList);
		}
	}

	@Override
	public List<MemberRankSetting> queryListByIsDefault(Boolean isDefault) {
		return memberRankSettingDao.queryListByIsDefault(isDefault);
	}

	//是否有购买资格
	public void buyCloudTeacherActive(Integer userId,Integer activeRemark,String remark){
		if(activeRemark == null || StringUtils.isEmpty(remark)){
			return;
		}
		if(activeRemark.equals(202109)){
			if(remark.equals("单月")){
				return;
			}
			StudentPaymentOrder studentPaymentOrder = studentPaymentOrderService.findByUserAndActive(userId,activeRemark,remark,null);
			if(studentPaymentOrder != null){
				DealStatusEnum status = studentPaymentOrder.getStatus();
				if(status == DealStatusEnum.SUCCESS){
					throw new BizException("您已参与此云教练活动，快去练习吧！");
				}
				if(status == DealStatusEnum.ING){
					throw new BizException("您有待处理的订单，请稍后重试");
				}
			}
		}
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class,isolation = Isolation.READ_COMMITTED)
	public Map buy(MemberPayParamDto memberPayParamDto) throws Exception {
		logger.info("云教练活动购买:memberPayParamDto:{} ",memberPayParamDto);
		Integer userId = memberPayParamDto.getUserId();
		BigDecimal amount = memberPayParamDto.getAmount();
		Integer memberRankId = memberPayParamDto.getMemberRankId();
		Student student = studentService.getLocked(userId);
		if (student == null) {
			throw new BizException("非法访问");
		}
		//是否可以购买
		buyCloudTeacherActive(userId,memberPayParamDto.getActiveRemark(),memberPayParamDto.getRemark());
		SysUser user = sysUserFeignService.queryUserById(userId);

		Integer organId = user.getOrganId();
		if (organId == null) {
			throw new BizException("学生没有分部编号");
		}

		MemberRankSetting memberRankSetting = memberRankSettingDao.get(memberRankId);
		if (memberRankSetting == null) {
			throw new BizException("会员等级错误");
		}

		MemberFeeSetting memberFeeSetting = memberFeeSettingService.findByRankIdAndOrganId(organId, memberRankId);

		if (memberFeeSetting == null) {
			throw new BizException("分部会员价格出现异常");
		}

		CloudTeacherOrder cloudTeacherOrder = new CloudTeacherOrder();

		BigDecimal actualAmount = BigDecimal.ZERO;

		switch (memberPayParamDto.getPeriodEnum()) {
			case DAY:
				actualAmount = memberFeeSetting.getCurrentDayFee();
				cloudTeacherOrder.setType(1);
				cloudTeacherOrder.setTime(1);
				break;
			case MONTH:
				actualAmount = memberFeeSetting.getCurrentMonthFee().multiply(new BigDecimal(memberPayParamDto.getBuyNum()));
				cloudTeacherOrder.setType(2);
				//购买月份加上赠送月份，目前只有按月购买的活动，其他的暂不处理
				cloudTeacherOrder.setTime(memberPayParamDto.getBuyNum() + memberPayParamDto.getGiveNum());
				break;
			case QUARTERLY:
				actualAmount = memberFeeSetting.getCurrentQuarterlyFee();
				cloudTeacherOrder.setType(2);
				cloudTeacherOrder.setTime(3);
				break;
			case YEAR_HALF:
				actualAmount = memberFeeSetting.getCurrentHalfYearFee();
				cloudTeacherOrder.setType(2);
				cloudTeacherOrder.setTime(6);
				break;
			case YEAR:
				actualAmount = memberFeeSetting.getCurrentYearFee();
				cloudTeacherOrder.setType(3);
				cloudTeacherOrder.setTime(1);
				break;

			default:
				break;
		}

		// 判断金额是否正确
		if (actualAmount.compareTo(amount) != 0) {
			throw new BizException("非法请求");
		}

		Date date = new Date();

		StudentPaymentOrder studentPaymentOrder = sysCouponCodeService.use(memberPayParamDto.getCouponIdList(),amount,true);
		amount = studentPaymentOrder.getActualAmount();
		studentPaymentOrder.setUserId(userId);
		studentPaymentOrder.setOrganId(organId);
		studentPaymentOrder.setRoutingOrganId(organId);
		studentPaymentOrder.setCreateTime(date);
		studentPaymentOrder.setUpdateTime(date);
		studentPaymentOrder.setBalancePaymentAmount(BigDecimal.ZERO);
		studentPaymentOrder.setStatus(DealStatusEnum.ING);
		studentPaymentOrder.setVersion(0);
		studentPaymentOrder.setType(OrderTypeEnum.MEMBER);
		studentPaymentOrder.setGroupType(GroupType.MEMBER);
		studentPaymentOrder.setRemitFee(BigDecimal.ZERO);
		studentPaymentOrder.setCourseRemitFee(BigDecimal.ZERO);
		studentPaymentOrder.setMemo(memberRankSetting.getName());
		studentPaymentOrder.setPaymentChannel("BALANCE");
		studentPaymentOrder.setMusicGroupId(memberRankId + "");

		if (memberPayParamDto.isUseBalance()) {
			SysUserCashAccount userCashAccount = sysUserCashAccountService.getLocked(userId);
			if (userCashAccount == null) {
				throw new BizException("用户账户找不到");
			}
			studentPaymentOrder.setPaymentChannel("BALANCE");
			if (userCashAccount.getBalance().subtract(amount).doubleValue() >= 0) {
				studentPaymentOrder.setActualAmount(BigDecimal.ZERO);
				studentPaymentOrder.setBalancePaymentAmount(amount);
				sysUserCashAccountService.updateBalance(userId, amount.negate(), PlatformCashAccountDetailTypeEnum.PAY_FEE, "购买会员");

			} else {
				if (userCashAccount.getBalance().doubleValue() > 0) {
					sysUserCashAccountService.updateBalance(userId, userCashAccount.getBalance().negate(), PlatformCashAccountDetailTypeEnum.PAY_FEE, "购买会员");
					studentPaymentOrder.setBalancePaymentAmount(userCashAccount.getBalance());
					studentPaymentOrder.setActualAmount(studentPaymentOrder.getExpectAmount().subtract(userCashAccount.getBalance()));
				}
			}
		}
		String orderNo = idGeneratorService.generatorId("payment") + "";
		studentPaymentOrder.setOrderNo(orderNo);
		studentPaymentOrder.setCreateTime(date);
		studentPaymentOrder.setUpdateTime(date);
		studentPaymentOrder.setVersion(0);
		studentPaymentOrderService.insert(studentPaymentOrder);

		// 订单明细
		cloudTeacherOrder.setOrganId(studentPaymentOrder.getOrganId());
		cloudTeacherOrder.setStudentId(studentPaymentOrder.getUserId());
		cloudTeacherOrder.setLevel(memberRankId);
		cloudTeacherOrder.setAmount(amount);
		cloudTeacherOrder.setStatus(0);
		cloudTeacherOrder.setOrderId(studentPaymentOrder.getId());
		cloudTeacherOrder.setCreateTime(date);
		cloudTeacherOrder.setUpdateTime(date);
		cloudTeacherOrder.setActiveRemark(memberPayParamDto.getActiveRemark());
		cloudTeacherOrder.setRemark(memberPayParamDto.getRemark());
		cloudTeacherOrderService.insert(cloudTeacherOrder);

		Map result = studentPaymentOrderService.createOrder(studentPaymentOrder, null, "BUY_MEMBER");

		return result;
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
	public boolean orderCallback(StudentPaymentOrder studentPaymentOrder) {

		Integer userId = studentPaymentOrder.getUserId();

		Student student = studentService.getLocked(userId);
		if (student == null) {
			throw new BizException("学员信息不存在");
		}
		
		//查询原定的是否已处理
		StudentPaymentOrder orignStudentPaymentOrder = studentPaymentOrderService.get(studentPaymentOrder.getId());
		if(orignStudentPaymentOrder == null){
			return false;
		}
		if(orignStudentPaymentOrder.getStatus() != DealStatusEnum.ING){
			return false;
		}
		
		Date nowDate = new Date();
		// 更新订单信息
		studentPaymentOrder.setUpdateTime(nowDate);
		studentPaymentOrderService.update(studentPaymentOrder);

		if (studentPaymentOrder.getStatus() == DealStatusEnum.CLOSE || studentPaymentOrder.getStatus() == DealStatusEnum.FAILED) {

			if (studentPaymentOrder.getBalancePaymentAmount() != null && studentPaymentOrder.getBalancePaymentAmount().compareTo(BigDecimal.ZERO) > 0) {
				sysUserCashAccountService.updateBalance(studentPaymentOrder.getUserId(), studentPaymentOrder.getBalancePaymentAmount(),
						PlatformCashAccountDetailTypeEnum.REFUNDS, "购买会员支付失败");
			}
			sysCouponCodeService.quit(studentPaymentOrder.getCouponCodeId());
			return true;
		}

		// 支付成功
		if (studentPaymentOrder.getStatus() == DealStatusEnum.SUCCESS) {
			// 查询会员订单信息
			CloudTeacherOrder cloudTeacherOrder = cloudTeacherOrderService.queryByOrderId(studentPaymentOrder.getId());
			cloudTeacherOrder.setStatus(2);
			cloudTeacherOrder.setUpdateTime(nowDate);

			PeriodEnum periodEnum = PeriodEnum.MONTH;
			switch (cloudTeacherOrder.getType()) {
			case 1:
				periodEnum = PeriodEnum.DAY;
				break;
			case 2:
				periodEnum = PeriodEnum.MONTH;
				break;
			case 3:
				periodEnum = PeriodEnum.YEAR;
				break;

			default:
				break;
			}

			if (student.getMemberRankSettingId() == null || nowDate.after(student.getMembershipEndTime())) {
				cloudTeacherOrder.setStartTime(nowDate);
				if (cloudTeacherOrder.getType() == 1) {
					cloudTeacherOrder.setEndTime(DateUtil.addDays(nowDate, cloudTeacherOrder.getTime()));
				} else if (cloudTeacherOrder.getType() == 2) {
					cloudTeacherOrder.setEndTime(DateUtil.addMonths(nowDate, cloudTeacherOrder.getTime()));
				} else if (cloudTeacherOrder.getType() == 3) {
					cloudTeacherOrder.setEndTime(DateUtil.addYears(nowDate, cloudTeacherOrder.getTime()));
				} else {
					throw new BizException("不支持的周期类型");
				}
			} else {
				cloudTeacherOrder.setStartTime(DateUtil.addDays(student.getMembershipEndTime(), 1));
				if (cloudTeacherOrder.getType() == 1) {
					cloudTeacherOrder.setEndTime(DateUtil.addDays(student.getMembershipEndTime(), cloudTeacherOrder.getTime()));
				} else if (cloudTeacherOrder.getType() == 2) {
					cloudTeacherOrder.setEndTime(DateUtil.addMonths(student.getMembershipEndTime(), cloudTeacherOrder.getTime()));
				} else if (cloudTeacherOrder.getType() == 3) {
					cloudTeacherOrder.setEndTime(DateUtil.addYears(student.getMembershipEndTime(), cloudTeacherOrder.getTime()));
				} else {
					throw new BizException("不支持的周期类型");
				}
			}

			cloudTeacherOrderService.update(cloudTeacherOrder);
			// 添加会员有效时长
			studentService.updateMemberRank(cloudTeacherOrder, periodEnum);
			//云教练活动赠送优惠券
			cloudTeacherActiveGiveCoupon(cloudTeacherOrder);
//			// 插入交易明细
			BigDecimal amount = studentPaymentOrder.getActualAmount();
			if (amount.compareTo(BigDecimal.ZERO) > 0) {
				SysUserCashAccount cashAccount = sysUserCashAccountService.get(userId);
				// 充值
				SysUserCashAccountDetail rechargeDetail = new SysUserCashAccountDetail();
				rechargeDetail.setAmount(amount);
				rechargeDetail.setBalance(cashAccount.getBalance().add(amount));
				rechargeDetail.setComment("缴费前充值");
				rechargeDetail.setCreateTime(nowDate);
				rechargeDetail.setStatus(DealStatusEnum.SUCCESS);
				rechargeDetail.setTransNo(studentPaymentOrder.getTransNo());
				rechargeDetail.setType(PlatformCashAccountDetailTypeEnum.RECHARGE);
				rechargeDetail.setUpdateTime(nowDate);
				rechargeDetail.setUserId(userId);
				rechargeDetail.setChannel(studentPaymentOrder.getPaymentChannel());
				rechargeDetail.setComAmount(studentPaymentOrder.getComAmount());
				rechargeDetail.setPerAmount(studentPaymentOrder.getPerAmount());
				sysUserCashAccountDetailService.insert(rechargeDetail);

				// 缴费
				SysUserCashAccountDetail paymentDetail = new SysUserCashAccountDetail();
				paymentDetail.setAmount(amount.negate());
				paymentDetail.setBalance(cashAccount.getBalance());
				paymentDetail.setComment("购买会员");
				paymentDetail.setCreateTime(nowDate);
				paymentDetail.setStatus(DealStatusEnum.SUCCESS);
				paymentDetail.setTransNo(studentPaymentOrder.getTransNo());
				paymentDetail.setType(PlatformCashAccountDetailTypeEnum.PAY_FEE);
				paymentDetail.setUpdateTime(nowDate);
				paymentDetail.setUserId(userId);
				rechargeDetail.setChannel(studentPaymentOrder.getPaymentChannel());
				sysUserCashAccountDetailService.insert(paymentDetail);
			}
//
			try {
				contractService.transferProduceContract(userId, null, CourseViewTypeEnum.MEMBER_FEE);
			} catch (Exception e) {
				logger.error("产品协议生成失败", e);
			}
		}
		return true;
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class,isolation = Isolation.READ_COMMITTED)
	public HttpResponseResult ActiveBuy(Integer userId, Integer memberRankId, PeriodEnum month, BigDecimal amount, boolean isUseBalance, Integer buyNum, int giveNum, int activeRemark, String remark,boolean repay) throws Exception {
		HttpResponseResult result = new HttpResponseResult();
		//判断用户是否已存在订单
		StudentPaymentOrder applyOrder = studentPaymentOrderService.findByUserAndActive(userId,202109,remark,"ING");
		if (applyOrder != null) {
			// 查询订单状态
			PayStatus payStatus = studentPaymentOrderService.queryPayStatus(applyOrder.getPaymentChannel(), applyOrder.getOrderNo(), applyOrder.getTransNo());
			if(payStatus == PayStatus.SUCCESSED){
				throw new BizException("订单已支付成功，请勿重复支付");
			}/*else if(payStatus == PayStatus.PAYING){
				throw new BizException("订单还在交易中，请稍后重试");
			}*/
			if(repay){
				//处理关闭订单
				applyOrder.setStatus(DealStatusEnum.CLOSE);
				applyOrder.setMemo("关闭云教练活动订单");
				if (applyOrder.getBalancePaymentAmount() != null && applyOrder.getBalancePaymentAmount().compareTo(BigDecimal.ZERO) > 0) {
					sysUserCashAccountService.updateBalance(applyOrder.getUserId(), applyOrder.getBalancePaymentAmount(),
							PlatformCashAccountDetailTypeEnum.REFUNDS, "购买云教练活动支付失败");
				}
				studentPaymentOrderService.update(applyOrder);
			}else {
				result.setCode(HttpStatus.CONTINUE.value());
				result.setMsg("您有待支付的订单");
				return result;
			}

		}
		result.setCode(HttpStatus.OK.value());
		MemberPayParamDto memberPayParamDto = new MemberPayParamDto();
		memberPayParamDto.setMemberRankId(memberRankId);
		memberPayParamDto.setUserId(userId);
		memberPayParamDto.setPeriodEnum(month);
		memberPayParamDto.setUseBalance(isUseBalance);
		memberPayParamDto.setBuyNum(buyNum);
		memberPayParamDto.setGiveNum(giveNum);
		memberPayParamDto.setActiveRemark(activeRemark);
		memberPayParamDto.setRemark(remark);
		memberPayParamDto.setAmount(amount);
		result.setData(this.buy(memberPayParamDto));
		return result;
	}

	//云教练活动赠送优惠券
	public void cloudTeacherActiveGiveCoupon(CloudTeacherOrder cloudTeacherOrder){
		Integer activeRemark = cloudTeacherOrder.getActiveRemark();
		String remark = cloudTeacherOrder.getRemark();
		if(activeRemark == null || StringUtils.isEmpty(remark)){
			return;
		}
		Integer studentId = cloudTeacherOrder.getStudentId();
		SysUser sysUser = sysUserFeignService.queryUserById(studentId);
		if(sysUser == null){
			throw new BizException("用户信息获取失败");
		}
		Integer couponId = 1;
		if(sysUser.getOrganId() != null && sysUser.getOrganId() == 4){
			couponId = 2;
		}
		if(activeRemark.equals(202109)){
			switch (remark){
				case "2赠1":
					sysCouponCodeService.exchangeCoupon(cloudTeacherOrder.getStudentId(),couponId,cloudTeacherOrder.getOrderId(),3);
					break;
				case "3赠2":
					sysCouponCodeService.exchangeCoupon(cloudTeacherOrder.getStudentId(),couponId,cloudTeacherOrder.getOrderId(),6);
					break;
			}
		}
	}
}
