package com.ym.mec.biz.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.ChargeTypeSubjectMapperDao;
import com.ym.mec.biz.dal.dao.GoodsDao;
import com.ym.mec.biz.dal.dao.MusicGroupPaymentCalenderCourseSettingsDao;
import com.ym.mec.biz.dal.dao.MusicGroupPaymentCalenderDao;
import com.ym.mec.biz.dal.dao.MusicGroupSubjectPlanDao;
import com.ym.mec.biz.dal.dao.OrganizationCloudTeacherFeeDao;
import com.ym.mec.biz.dal.dao.StudentPaymentOrderDetailDao;
import com.ym.mec.biz.dal.dto.MusicGroupGoodsAndDiscountDto;
import com.ym.mec.biz.dal.dto.MusicGroupRegRespDto;
import com.ym.mec.biz.dal.dto.MusicGroupSubjectGoodsAndInfoDto;
import com.ym.mec.biz.dal.entity.ChargeTypeSubjectMapper;
import com.ym.mec.biz.dal.entity.CourseSchedule.CourseScheduleType;
import com.ym.mec.biz.dal.entity.Goods;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalenderCourseSettings;
import com.ym.mec.biz.dal.entity.MusicGroupSubjectGoodsGroup;
import com.ym.mec.biz.dal.entity.MusicGroupSubjectPlan;
import com.ym.mec.biz.dal.entity.OrganizationCloudTeacherFee;
import com.ym.mec.biz.dal.entity.OrganizationCourseUnitPriceSettings;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.enums.CourseViewTypeEnum;
import com.ym.mec.biz.dal.enums.DealStatusEnum;
import com.ym.mec.biz.dal.enums.GoodsType;
import com.ym.mec.biz.service.GoodsService;
import com.ym.mec.biz.service.MemberRankPrivilegesService;
import com.ym.mec.biz.service.MusicGroupService;
import com.ym.mec.biz.service.MusicGroupSubjectGoodsGroupService;
import com.ym.mec.biz.service.MusicGroupSubjectPlanService;
import com.ym.mec.biz.service.OrganizationCourseUnitPriceSettingsService;
import com.ym.mec.biz.service.StudentRegistrationService;
import com.ym.mec.biz.service.SubjectService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;

@Service
public class MusicGroupSubjectPlanServiceImpl extends BaseServiceImpl<Integer, MusicGroupSubjectPlan> implements MusicGroupSubjectPlanService {

    @Autowired
    private MusicGroupSubjectPlanDao musicGroupSubjectPlanDao;

    @Autowired
    private StudentRegistrationService studentRegistrationService;

    @Autowired
    private GoodsService goodsService;

    @Autowired
    private MusicGroupSubjectGoodsGroupService musicGroupSubjectGoodsGroupService;

    @Autowired
    private MusicGroupService musicGroupService;

    @Autowired
    private SubjectService subjectService;
    @Autowired
    private StudentPaymentOrderDetailDao studentPaymentOrderDetailDao;
    @Autowired
    private GoodsDao goodsDao;
    @Autowired
    private ChargeTypeSubjectMapperDao chargeTypeSubjectMapperDao;
    @Autowired
    private MusicGroupPaymentCalenderCourseSettingsDao musicGroupPaymentCalenderCourseSettingsDao;
    @Autowired
    private MusicGroupPaymentCalenderDao musicGroupPaymentCalenderDao;
    @Autowired
    private OrganizationCloudTeacherFeeDao organizationCloudTeacherFeeDao;

	@Autowired
	private MemberRankPrivilegesService memberRankPrivilegesService;
	
	@Autowired
	private OrganizationCourseUnitPriceSettingsService organizationCourseUnitPriceSettingsService;
	
    @Resource
    private SysUserFeignService sysUserFeignService;

    @Override
    public BaseDAO<Integer, MusicGroupSubjectPlan> getDAO() {
        return musicGroupSubjectPlanDao;
    }

    @Override
    public List<MusicGroupSubjectPlan> getMusicSubjectClassPlan(String musicGroupId) {
        List<MusicGroupSubjectPlan> musicSubjectClassPlans = musicGroupSubjectPlanDao.getMusicSubjectClassPlan(musicGroupId);
        List<MusicGroupSubjectPlan> noClassStuCounts = studentRegistrationService.getNoClassStuCountByMusicGroupId(musicGroupId);

        for (MusicGroupSubjectPlan musicSubjectClassPlan : musicSubjectClassPlans) {
            for (MusicGroupSubjectPlan noClassStuCount : noClassStuCounts) {
                if (!noClassStuCount.getSubjectId().equals(musicSubjectClassPlan.getSubjectId())) continue;
                musicSubjectClassPlan.setNoClassStudentNum(noClassStuCount.getNoClassStudentNum());
            }
            if (musicSubjectClassPlan.getNoClassStudentNum() == null) {
                musicSubjectClassPlan.setNoClassStudentNum(0);
            }
        }
        return musicSubjectClassPlans;
    }

    @Override
    public MusicGroupSubjectPlan getMusicOneSubjectClassPlan(String musicGroupId, int subjectId) {
        return musicGroupSubjectPlanDao.getMusicOneSubjectClassPlan(musicGroupId, subjectId);
    }

    @Override
    public MusicGroupSubjectGoodsAndInfoDto getSubjectGoodsAndInfo(String musicGroupId, Integer subjectId) {
        MusicGroup musicGroup = musicGroupService.get(musicGroupId);
        
        if(musicGroup == null){
        	throw new BizException("乐团信息查询失败");
        }

        //课程形态
        Map<String, Object> courseForm = JSON.parseObject(musicGroup.getCourseForm(), Map.class);

        //新的课程形态10.27
        MusicGroupPaymentCalender musicGroupRegCalender = musicGroupPaymentCalenderDao.getMusicGroupRegCalender(musicGroupId);
        if (musicGroupRegCalender != null) {
            List<MusicGroupPaymentCalenderCourseSettings> courseSettings = musicGroupPaymentCalenderCourseSettingsDao.getMusicGroupRegCalenderCourseSettings(musicGroupId);
            musicGroupRegCalender.setMusicGroupPaymentCalenderCourseSettingsList(courseSettings);
        }

        //乐团计划及收费信息
        MusicGroupSubjectPlan musicOneSubjectClassPlan = this.getMusicOneSubjectClassPlan(musicGroupId, subjectId);

        //乐团乐器及辅件信息
        List<MusicGroupSubjectGoodsGroup> goodsGroups = musicGroupSubjectGoodsGroupService.findGoodsGroup(musicGroupId, subjectId);

        for (MusicGroupSubjectGoodsGroup goodsGroup : goodsGroups) {
            String childGoodIds = "";
            for (Goods goods : goodsGroup.getGoodsList()) {
                if (StringUtils.isNotBlank(goods.getComplementGoodsIdList())) {
                    childGoodIds += StringUtils.isNotBlank(childGoodIds) ? "," : "" + goods.getComplementGoodsIdList();
                }
            }
            if (StringUtils.isNotBlank(childGoodIds)) {
                goodsGroup.setChildGoodsList(goodsDao.findGoodsByIds(childGoodIds));
            }
        }


        //获取声部（科目）下其他商品
        MusicGroupSubjectGoodsAndInfoDto musicGroupSubjectGoodsAndInfo = new MusicGroupSubjectGoodsAndInfoDto();
        musicGroupSubjectGoodsAndInfo.setMusicGroupSubjectPlan(musicOneSubjectClassPlan);
        musicGroupSubjectGoodsAndInfo.setCourseScheduleInfo(courseForm);
        musicGroupSubjectGoodsAndInfo.setMusicGroupPaymentCalender(musicGroupRegCalender);
        musicGroupSubjectGoodsAndInfo.setMusicGroupSubjectGoodsGroupList(goodsGroups);
        musicGroupSubjectGoodsAndInfo.setMusicGroup(musicGroup);
		if (musicGroup.getCourseViewType().equals(CourseViewTypeEnum.MEMBER_FEE) || musicGroup.getCloudTeacherType() != null) {
			// 历史数据
			if (musicGroup.getCloudTeacherType() != null) {
				OrganizationCloudTeacherFee cloudTeacherFee = organizationCloudTeacherFeeDao.getByOrganId(musicGroup.getOrganId());
				if (musicGroup.getCloudTeacherType() == 0) {
					musicGroupSubjectGoodsAndInfo.setCloudTeacherFee(cloudTeacherFee.getPrice());
				} else if (musicGroup.getCloudTeacherType() == 1) {
					musicGroupSubjectGoodsAndInfo.setCloudTeacherFee(cloudTeacherFee.getPlusPrice());
				}
			} else {
				musicGroupSubjectGoodsAndInfo.setCloudTeacherFee(musicGroupRegCalender.getMemberPaymentAmount());
			}

			if (musicGroupRegCalender != null && musicGroupRegCalender.getMemberRankSettingId() != null) {
				musicGroupSubjectGoodsAndInfo.setMemberPrivilegesItemList(memberRankPrivilegesService.queryByMemberRankId(musicGroupRegCalender
						.getMemberRankSettingId()));
			} else{
				musicGroupSubjectGoodsAndInfo.setMemberPrivilegesItemList(memberRankPrivilegesService.queryByMemberRankId(1));
			}
			
			Map<String, BigDecimal> memberCoursePrice = new HashMap<String, BigDecimal>();
			
			OrganizationCourseUnitPriceSettings singleUnitPriceSettings = organizationCourseUnitPriceSettingsService.queryByOrganIdAndCourseTypeAndChargeType(musicGroup.getOrganId(), CourseScheduleType.SINGLE, musicGroup.getChargeTypeId());
			if (singleUnitPriceSettings != null) {
				memberCoursePrice.put(CourseScheduleType.SINGLE.name(),
						singleUnitPriceSettings.getUnitPrice().multiply(new BigDecimal(1980)).setScale(0, BigDecimal.ROUND_HALF_UP));
			}
			OrganizationCourseUnitPriceSettings mixUnitPriceSettings = organizationCourseUnitPriceSettingsService.queryByOrganIdAndCourseTypeAndChargeType(musicGroup.getOrganId(), CourseScheduleType.MIX, musicGroup.getChargeTypeId());
			if (mixUnitPriceSettings != null) {
				memberCoursePrice.put(CourseScheduleType.MIX.name(),
						mixUnitPriceSettings.getUnitPrice().multiply(new BigDecimal(1980)).setScale(0, BigDecimal.ROUND_HALF_UP));
			}
			
			musicGroupSubjectGoodsAndInfo.setMemberCoursePrice(memberCoursePrice);
		}
		
		// 查询已购买内容
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if(sysUser != null){
        	Integer userId = sysUser.getId();
        	musicGroupSubjectGoodsAndInfo.setStudentPaymentOrderDetailList(studentPaymentOrderDetailDao.findUserApplyOrder(userId, musicGroupId, DealStatusEnum.SUCCESS));
        }
        
        return musicGroupSubjectGoodsAndInfo;
    }

    @Override
    public MusicGroupRegRespDto getMusicGroupRegInfo(String musicGroupId) {
        MusicGroupRegRespDto musicGroupRegResp = new MusicGroupRegRespDto();

        //乐团信息
        MusicGroup musicGroup = musicGroupService.get(musicGroupId);
        musicGroupRegResp.setMusicGroupName(musicGroup.getName());
        musicGroupRegResp.setCourseViewType(musicGroup.getCourseViewType());
        musicGroupRegResp.setMusicGroupStatus(musicGroup.getStatus().getCode());
        //声部信息
        List<Subject> subjectList = subjectService.findSubByMusicGroupId(musicGroupId);
        musicGroupRegResp.setSubjectList(subjectList);

        return musicGroupRegResp;
    }

    @Override
    public void addApplyStudentNum(String musicGroupId, Integer subjectId, int num) {
        boolean flag = false;
        for (int i = 10; i >= 1; i--) {
            MusicGroupSubjectPlan musicOneSubjectClassPlan = musicGroupSubjectPlanDao.getMusicOneSubjectClassPlan(musicGroupId, subjectId);
            if (musicOneSubjectClassPlan == null) {
                MusicGroup musicGroup = musicGroupService.get(musicGroupId);
                Subject subject = subjectService.get(subjectId);
                throw new BizException("操作失败：{}乐团缺少{}声部", musicGroup.getName(), subject.getName());
            }
            int applyNum = musicOneSubjectClassPlan.getApplyStudentNum() == null ? 0 : musicOneSubjectClassPlan.getApplyStudentNum();
            musicOneSubjectClassPlan.setApplyStudentNum(applyNum + num);
            musicOneSubjectClassPlan.setUpdateTime(new Date());
            int updateCount = musicGroupSubjectPlanDao.update(musicOneSubjectClassPlan);
            if (updateCount >= 0) {
                flag = true;
                break;
            }
        }
        if (!flag) {
            throw new BizException("系统繁忙，请重试");
        }
    }

    @Override
    public int batchUpdateFee(List<MusicGroupSubjectPlan> musicGroupSubjectPlans) {
        return musicGroupSubjectPlanDao.batchUpdateFee(musicGroupSubjectPlans);
    }

    @Override
    public MusicGroupSubjectPlan findSubjectPlan(String musicGroupId, Integer subjectId) {
        return musicGroupSubjectPlanDao.findSubjectPlan(musicGroupId, subjectId);
    }

    @Override
    public List<MusicGroupGoodsAndDiscountDto> getSubjectGoods(Integer subjectId, String type, Integer chargeTypeId, Integer courseViewType, String musicGroupId) {
        Integer organId = null;
        if (StringUtils.isNotEmpty(musicGroupId)) {
            MusicGroup musicGroup = musicGroupService.get(musicGroupId);
            courseViewType = musicGroup.getCourseViewType().getCode();
            organId = musicGroup.getOrganId();
        }
        List<MusicGroupGoodsAndDiscountDto> goodsList = goodsDao.getMusicGroupGoodsAndDiscount(subjectId, type, courseViewType,organId);
        if (chargeTypeId == null || (type != null && !"INSTRUMENT".equals(type))) {
            return goodsList;
        }

        ChargeTypeSubjectMapper subjectDiscount = chargeTypeSubjectMapperDao.getSubjectDiscount(chargeTypeId, subjectId);
        if (subjectDiscount == null || subjectDiscount.getGoodsDiscountRate() == null || subjectDiscount.getGoodsDiscountRate().compareTo(new BigDecimal(100)) == 0) {
            return goodsList;
        }
        for (MusicGroupGoodsAndDiscountDto musicGroupGoodsAndDiscountDto : goodsList) {
            if (musicGroupGoodsAndDiscountDto.getType() != null && musicGroupGoodsAndDiscountDto.getType().equals(GoodsType.INSTRUMENT)) {
                musicGroupGoodsAndDiscountDto.setDiscountRate(subjectDiscount.getGoodsDiscountRate());
                musicGroupGoodsAndDiscountDto.setGroupPurchasePrice(musicGroupGoodsAndDiscountDto.getDiscountPrice().multiply(subjectDiscount.getGoodsDiscountRate()).divide(new BigDecimal(100), 0, BigDecimal.ROUND_HALF_UP));
            }
        }
        return goodsList;
    }
}
