package com.ym.mec.biz.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.service.*;

import com.ym.mec.util.excel.POIUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSON;
import com.ym.mec.biz.dal.dto.PageInfoOrder;
import com.ym.mec.biz.dal.dto.SporadicChargeInfoDto;
import com.ym.mec.biz.dal.dto.StudentPaymentOrderDto;
import com.ym.mec.biz.dal.dto.StudentPaymentOrderExportDto;
import com.ym.mec.biz.dal.page.SporadicOrderQueryInfo;
import com.ym.mec.biz.dal.page.StudentPaymentOrderQueryInfo;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.adapay.ConfigInit;
import com.ym.mec.thirdparty.adapay.Payment;
import com.ym.mec.thirdparty.yqpay.Msg;
import com.ym.mec.thirdparty.yqpay.RsqMsg;
import com.ym.mec.thirdparty.yqpay.YqPayFeignService;
import com.ym.mec.thirdparty.yqpay.YqPayUtil;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;

@Service
public class StudentPaymentOrderServiceImpl extends BaseServiceImpl<Long, StudentPaymentOrder> implements StudentPaymentOrderService {

    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;
    @Autowired
    private YqPayFeignService yqPayFeignService;
    @Autowired
    private StudentRegistrationService studentRegistrationService;
    @Autowired
    @Lazy
    private VipGroupService vipGroupService;
    @Autowired
    private MusicGroupService musicGroupService;
    @Autowired
    private SporadicChargeInfoService sporadicChargeInfoService;
    @Autowired
    private PracticeGroupService practiceGroupService;
    @Autowired
    private SysUserCashAccountDao sysUserCashAccountDao;
    @Autowired
    private StudentRepairService studentRepairService;
    @Autowired
    private SysUserCashAccountService sysUserCashAccountService;
    @Autowired
    private SubjectChangeService subjectChangeService;
    @Autowired
    private DegreeRegistrationService degreeRegistrationService;
    @Autowired
    private StudentPaymentRouteOrderDao studentPaymentRouteOrderDao;

    @Autowired
    private StudentPaymentOrderDetailService studentPaymentOrderDetailService;

    @Autowired
    private IdGeneratorService idGeneratorService;
    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private GoodsDao goodsDao;
    @Autowired
    private StudentInstrumentService studentInstrumentService;
    @Autowired
    private ReplacementInstrumentActivityService replacementInstrumentActivityService;
    @Autowired
    private ChildrenDayReserveService childrenDayReserveService;

    @Autowired
    private MemberRankSettingService memberRankSettingService;
    @Autowired
    private PayService payService;
    @Autowired
    private StudentPaymentRouteOrderService studentPaymentRouteOrderService;

    @Autowired
    private CloudTeacherOrderService cloudTeacherOrderService;
    @Autowired
    private SysCouponCodeService sysCouponCodeService;
    @Autowired
    private VipGroupActivityDao vipGroupActivityDao;
    @Autowired
    private ActivityUserMapperDao activityUserMapperDao;
    @Autowired
    private StudentService studentService;

    @Override
    public BaseDAO<Long, StudentPaymentOrder> getDAO() {
        return studentPaymentOrderDao;
    }

    private final static Logger LOGGER = LoggerFactory.getLogger(StudentPaymentOrderServiceImpl.class);

    @Override
    public StudentPaymentOrder findByUserAndActive(Integer userId, Integer activeRemark, String remark,String status) {
        return studentPaymentOrderDao.findByUserAndActive(userId,activeRemark,remark,status);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
	public Map createOrder(StudentPaymentOrder studentPaymentOrder, List<StudentPaymentOrderDetail> details, String payReceiver) throws Exception {

    	BigDecimal cashAmount = studentPaymentOrder.getActualAmount();
    	BigDecimal expectAmount = studentPaymentOrder.getExpectAmount();
    	BigDecimal balance = studentPaymentOrder.getBalancePaymentAmount();

		if (cashAmount.add(balance).compareTo(expectAmount) != 0) {
			throw new BizException("支付金额不正确，请重试");
		}

		Date date = new Date();
		studentPaymentOrder.setUpdateTime(date);

		StudentPaymentOrder orignStudentPaymentOrder = get(studentPaymentOrder.getId());
		if(orignStudentPaymentOrder == null){

			String orderNo = idGeneratorService.generatorId("payment") + "";
			studentPaymentOrder.setOrderNo(orderNo);
			studentPaymentOrder.setCreateTime(date);
			studentPaymentOrder.setUpdateTime(date);
			studentPaymentOrder.setVersion(0);
			insert(studentPaymentOrder);
		}

		if(details != null && details.size() > 0){
			for(StudentPaymentOrderDetail spod : details){
				spod.setPaymentOrderId(studentPaymentOrder.getId());
				spod.setCreateTime(date);
				spod.setUpdateTime(date);
			}

			studentPaymentOrderDetailService.batchAdd(details,studentPaymentOrder.getCouponRemitFee());
		}

		String baseApiUrl = sysConfigDao.findConfigValue("base_api_url");

		if (cashAmount.compareTo(BigDecimal.ZERO) == 0) {

			studentPaymentRouteOrderService.addRouteOrder(studentPaymentOrder.getOrderNo(), studentPaymentOrder.getOrganId(), balance);

			Map<String, String> notifyMap = new HashMap<>(4);
			notifyMap.put("tradeState", "1");
			notifyMap.put("merOrderNo", studentPaymentOrder.getOrderNo());
			notifyMap.put("channelType", "");
			notifyMap.put("orderNo", "");
			updateOrder(notifyMap);
			return notifyMap;
		}

		Map<String, Object> payMap = payService.getPayMap(cashAmount, balance, studentPaymentOrder.getOrderNo(), baseApiUrl + "/api-student/studentOrder/notify", baseApiUrl
				+ "/api-student/studentOrder/paymentResult?orderNo=" + studentPaymentOrder.getOrderNo(), studentPaymentOrder.getMemo(), studentPaymentOrder.getMemo(), studentPaymentOrder.getOrganId(), payReceiver);

		studentPaymentOrder.setMerNos((String) payMap.get("routingMerNos"));
		studentPaymentOrder.setPaymentChannel((String) payMap.get("type"));
		studentPaymentOrder.setUpdateTime(date);
		update(studentPaymentOrder);
		return payMap;
	}

	@Override
    public List<Goods> queryApplyGoodsList(String musicGroupId, OrderDetailTypeEnum type) {
        return studentPaymentOrderDao.queryApplyGoodsList(musicGroupId, type);
    }

    @Override
    public List<StudentPaymentOrder> queryByCondition(GroupType groupType, String musicGroupId, Integer userId, DealStatusEnum status, OrderTypeEnum type) {
        return studentPaymentOrderDao.queryByCondition(groupType, musicGroupId, userId, status, type);
    }

    @Override
    public List<StudentPaymentOrder> findMusicGroupApplyOrderByStatus(Integer userId, String musicGroupId, DealStatusEnum status) {
        return studentPaymentOrderDao.findMusicGroupApplyOrderByStatus(userId, musicGroupId, status);
    }

    @Override
    public StudentPaymentOrder findOrderByOrderNo(String orderNo) {
        return studentPaymentOrderDao.findOrderByOrderNo(orderNo);
    }

    @Override
	public List<StudentPaymentOrder> queryByBatchNo(Integer userId, String batchNo, DealStatusEnum status) {
		return studentPaymentOrderDao.queryByBatchNo(userId, batchNo, status);
	}

	@Override
    public int findPayOrderNum() {
        return studentPaymentOrderDao.findPayOrderNum();
    }

    @Override
    public List<StudentPaymentOrder> findOrdersByStatus(DealStatusEnum status, String paymentChannel) {
        return studentPaymentOrderDao.findOrdersByStatus(status, paymentChannel);
    }

    @Override
    public List<StudentPaymentOrder> findOrdersOverTime(List<String> orderNoList, DealStatusEnum status, Date beforeTime) {
        return studentPaymentOrderDao.findOrdersOverTime(orderNoList, status, beforeTime);
    }

    @Override
    public void queryOrderStatus() throws Exception {
        yqPayQuery(null);
        adaPayQuery(null);
        reConfirmOrder(null, null);
    }

    private void yqPayQuery(List<StudentPaymentOrder> payingOrders) throws Exception {
        if (payingOrders == null) {
            payingOrders = findOrdersByStatus(DealStatusEnum.ING, "YQPAY");
        }
        if (payingOrders.size() == 0) {
            return;
        }
        List<String> orderNoList = payingOrders.stream().map(StudentPaymentOrder::getOrderNo).collect(Collectors.toList());
        String merOrderNos = payingOrders.stream().map(StudentPaymentOrder::getOrderNo).collect(Collectors.joining(","));

        String notifyUrl = ""; //回调地址
        Map<String, Object> resultMap = new LinkedHashMap<>();
        resultMap.put("merOrderNoList", merOrderNos);
        Map<String, Object> requestMap = YqPayUtil.getRequestMap(notifyUrl, resultMap);

        RsqMsg rsqMsg = new RsqMsg(requestMap);

        Msg queryRs = yqPayFeignService.orderQuery(rsqMsg);

        if (queryRs.getCode().equals("88")) {
            //更新订单状态
            String[] statusArr = {"0", "1", "7"};
            String responseParameters = queryRs.getResponseParameters();
            List<Map<String, String>> responseList = JSON.parseObject(responseParameters, List.class);
            for (Map<String, String> response : responseList) {
                Map<String, String> rpMap = response;
                String channelType = rpMap.get("channelType").equals("1") ? "WXPay" : (rpMap.get("channelType").equals("2") ? "Alipay" : "quickPay");
                rpMap.put("channelType", channelType);

                if (orderNoList.contains(rpMap.get("merOrderNo"))) {
                    orderNoList.remove(rpMap.get("merOrderNo"));
                }

                if (Arrays.asList(statusArr).contains(rpMap.get("tradeState"))) {
                    try {
                        updateOrder(rpMap); //更新订单
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    continue;
                }
            }
            closeOrders(orderNoList); //关闭订单
        }
    }

    private void adaPayQuery(List<StudentPaymentOrder> payingOrders) throws Exception {
        if (payingOrders == null) {
            payingOrders = findOrdersByStatus(DealStatusEnum.ING, "ADAPAY");
        }
        
        Date date = new Date();
        
        if (payingOrders.size() == 0) {
            return;
        }
        List<String> orderNoList = new ArrayList<String>();

        for (StudentPaymentOrder payingOrder : payingOrders) {
            //if (payingOrder.getTransNo() == null && "205".equals(payingOrder.getPaymentAccountNo()) && DateUtil.isSameDay(payingOrder.getCreateTime(), new Date())) {
            if (payingOrder.getTransNo() == null && "205".equals(payingOrder.getPaymentAccountNo())) {
                continue;
            }
            if (payingOrder.getTransNo() == null) {
                orderNoList.add(payingOrder.getOrderNo());
                continue;
            }
            //查询支付超过10分钟的
            if(DateUtil.minutesBetween(payingOrder.getCreateTime(), date) < 10){
            	continue;
            }
            Map<String, Object> payment = Payment.queryPayment(payingOrder.getTransNo());
            LOGGER.info("订单[{}]交易主动查询接口返回：{}", payingOrder.getTransNo(), payment);
            
            Map<String, String> rpMap = new HashMap<>();
            rpMap.put("merOrderNo", payingOrder.getOrderNo());
            rpMap.put("orderNo", payingOrder.getTransNo());
            rpMap.put("channelType", (String) payment.get("pay_channel"));
            String status = (String) payment.get("status");
            if (payment.containsKey("error_msg")) {
                rpMap.put("remarks", (String) payment.get("error_msg"));
            }
            if (status.equals("succeeded")) {
                rpMap.put("tradeState", "1");
            }else if (status.equals("failed") || (DateUtil.minutesBetween(DateUtil.addHours(payingOrder.getUpdateTime(), 2), date) > 0)) {
                rpMap.put("tradeState", "0");
            }

            if (status.equals("succeeded") ||
                    (status.equals("failed") || payingOrder.getCreateTime().before(DateUtil.addMinutes(new Date(), -35)))) {
                try {
                    updateOrder(rpMap); //更新订单
                } catch (Exception e) {
                    e.printStackTrace();
                    continue;
                }
            }
        }
        closeOrders(orderNoList);
    }

    @Transactional(rollbackFor = Exception.class)
    public void updateOrder(Map<String, String> rpMap) throws Exception {
        DealStatusEnum status = rpMap.get("tradeState").equals("1") ? DealStatusEnum.SUCCESS : DealStatusEnum.FAILED;
        StudentPaymentOrder order = findOrderByOrderNo(rpMap.get("merOrderNo"));
        //关闭或失败的订单查询订单成功,订单改成成功，钱退到余额
        if (order != null && (order.getStatus().equals(DealStatusEnum.CLOSE) || order.getStatus().equals(DealStatusEnum.FAILED)) && status.equals(DealStatusEnum.SUCCESS)) {
            String memo = order.getStatus().equals(DealStatusEnum.CLOSE) ? "关闭订单" : "失败订单";
            memo = memo + ",实际支付成功,退到用户余额";
            //更新订单状态
            order.setStatus(status);
            order.setTransNo(rpMap.get("orderNo"));
            order.setPaymentBusinessChannel(rpMap.get("channelType"));
            order.setPayTime(new Date());
            order.setMemo(memo);
            int updateCount = this.update(order);
            if (updateCount <= 0) {
                throw new BizException("订单更新失败");
            }
            //增加用户余额
            sysUserCashAccountService.updateBalance(order.getUserId(), order.getActualAmount(),
                    PlatformCashAccountDetailTypeEnum.REFUNDS, memo + ",订单号:" + order.getOrderNo());

            //退还优惠券
            sysCouponCodeService.quit(order.getCouponCodeId());
            if (StringUtils.isNotBlank(order.getPaymentChannel())
                    && order.getPaymentChannel().equals("ADAPAY")
                    && !rpMap.containsKey("simulation")) {
                confirmOrder(order);
            }
            return;
        }

        if (order == null || !order.getStatus().equals(DealStatusEnum.ING)) {
            return;
        }

        if (status.equals(DealStatusEnum.SUCCESS)) {
            order.setPayTime(new Date());
        } else {
            order.setMemo(rpMap.get("remarks"));
        }
        order.setStatus(status);
        if (StringUtils.isBlank(order.getTransNo())) {
            order.setTransNo(rpMap.get("orderNo"));
        }
        order.setPaymentBusinessChannel(rpMap.get("channelType"));

        callOrderCallBack(order);

        if (status.equals(DealStatusEnum.SUCCESS)
                && StringUtils.isNotBlank(order.getPaymentChannel())
                && order.getPaymentChannel().equals("ADAPAY")
                && !rpMap.containsKey("simulation")) {
            confirmOrder(order);
        }
    }

    private Boolean confirmOrder(StudentPaymentOrder order) {
    	Date date = new Date();
        List<StudentPaymentRouteOrder> routeOrders = studentPaymentRouteOrderDao.getRouteOrders(order.getOrderNo());
        for (StudentPaymentRouteOrder routeOrder : routeOrders) {
            Map<String, Object> divMember = new HashMap<>();
            divMember.put("member_id", routeOrder.getMerNo());//分佣账户
            divMember.put("amount", routeOrder.getRouteAmount().setScale(2, BigDecimal.ROUND_HALF_UP));//分佣金额
            divMember.put("fee_flag", "Y");
            if (routeOrder.getMerNo().equals(ConfigInit.merNo)) {
                divMember.put("member_id", 0);
            }
            List<Map<String, Object>> divMembers = new ArrayList<>();
            divMembers.add(divMember);

            Map<String, Object> confirm = new HashMap<>();
            confirm.put("payment_id", order.getTransNo());
            confirm.put("order_no", idGeneratorService.generatorId("payment"));
            confirm.put("confirm_amt", routeOrder.getRouteAmount());
            confirm.put("div_members", divMembers);
            try {
                Map<String, Object> map = Payment.createConfirm(confirm);
                LOGGER.info("分账信息返回：{}",map);
                routeOrder.setServiceFee(new BigDecimal(map.get("fee_amt").toString()));
                routeOrder.setUpdateTime(date);
                studentPaymentRouteOrderDao.update(routeOrder);
            } catch (Exception e) {
                continue;
            }
        }
        return true;
    }

    //Date date = new Date();
    //        BigDecimal balance = BigDecimal.ZERO;
    //        MusicGroup musicGroup = musicGroupDao.get(studentRegistration.getMusicGroupId());
    //        if (registerPayDto.getIsUseBalancePayment() && amount.compareTo(BigDecimal.ZERO) > 0) {
    //            SysUserCashAccount userCashAccount = sysUserCashAccountService.getLocked(userId);
    //            if (userCashAccount == null) {
    //                throw new BizException("用户账户找不到");
    //            }
    //            if (userCashAccount.getBalance() != null && userCashAccount.getBalance().compareTo(BigDecimal.ZERO) > 0) {
    //                balance = amount.compareTo(userCashAccount.getBalance()) >= 0 ? userCashAccount.getBalance() : amount;
    //                amount = amount.subtract(balance);
    //                studentPaymentOrder.setPaymentChannel("BALANCE");
    //                studentPaymentOrder.setActualAmount(amount);
    //                studentPaymentOrder.setBalancePaymentAmount(balance);
    //                sysUserCashAccountService.updateBalance(userId, balance.negate(), PlatformCashAccountDetailTypeEnum.PAY_FEE, "乐团报名");
    //            }
    //        }
    //        studentPaymentOrder.setPaymentChannel("BALANCE");
    //        studentPaymentOrder.setRemitFee(remitFee);
    //        studentPaymentOrder.setCourseRemitFee(courseRemitFee);
    //        studentPaymentOrder.setOrganId(musicGroup.getOrganId());
    //        studentPaymentOrder.setRoutingOrganId(musicGroup.getOrganId());
    //        studentPaymentOrder.setUpdateTime(date);
    //        studentPaymentOrderService.update(studentPaymentOrder);
    //        studentPaymentOrder.setVersion(studentPaymentOrder.getVersion() + 1);
    //
    //        if (amount.compareTo(BigDecimal.ZERO) == 0) {
    //            //如果赠送网管课
    //            studentPaymentRouteOrderService.addRouteOrder(orderNo, musicGroup.getOrganId(), balance);
    //            Map<String, String> notifyMap = new HashMap<>();
    //            notifyMap.put("tradeState", "1");
    //            notifyMap.put("merOrderNo", studentPaymentOrder.getOrderNo());
    //            notifyMap.put("channelType", channelType);
    //            notifyMap.put("orderNo", "");
    //            studentPaymentOrderService.updateOrder(notifyMap);
    //            return notifyMap;
    //        }

    //调用相应业务回调接口
    public void callOrderCallBack(StudentPaymentOrder order) throws Exception {
        if (order.getType().equals(OrderTypeEnum.APPLY)) { //报名订单
            studentRegistrationService.updateApplyOrder(order);
        } else if (order.getType().equals(OrderTypeEnum.SMALL_CLASS_TO_BUY)) {
            vipGroupService.orderCallback(order);
        } else if (order.getType().equals(OrderTypeEnum.RENEW) || order.getType().equals(OrderTypeEnum.ADD_STUDENT)) {
            musicGroupService.renewForCallback(order);
        } else if (order.getType().equals(OrderTypeEnum.SPORADIC) || order.getType().equals(OrderTypeEnum.LUCK) || order.getType().equals(OrderTypeEnum.DOUBLE_ELEVEN2020)) {
            sporadicChargeInfoService.renewForCallback(order);
        } else if (order.getType().equals(OrderTypeEnum.PRACTICE_GROUP_BUY) || order.getType().equals(OrderTypeEnum.PRACTICE_GROUP_RENEW)) {
            practiceGroupService.orderCallback(order);
        } else if (order.getType().equals(OrderTypeEnum.REPAIR)) {
            studentRepairService.orderCallback(order);
        } else if (order.getType().equals(OrderTypeEnum.GOODS_SELL)) {
            studentRepairService.goodsSellOrderCallback(order);
        } else if (order.getType().equals(OrderTypeEnum.SUBJECT_CHANGE)) {
            subjectChangeService.orderCallback(order);
        } else if (order.getType().equals(OrderTypeEnum.DEGREE_REGISTRATION)) {
            degreeRegistrationService.updateStatus(order);
        } else if (order.getType().equals(OrderTypeEnum.MAINTENANCE)) {
            studentInstrumentService.orderCallback(order);
        } else if (order.getType().equals(OrderTypeEnum.REPLACEMENT)) {
            replacementInstrumentActivityService.orderCallback(order);
        }else if (order.getType().equals(OrderTypeEnum.DEGREE)) {
            childrenDayReserveService.orderCallback(order);
        }else if (order.getType().equals(OrderTypeEnum.MEMBER)) {
        	memberRankSettingService.orderCallback(order);
        }else if (order.getType().equals(OrderTypeEnum.ACTIVITY)) {
            practiceGroupService.activityOrderCallback(order);
        }else if (order.getType().equals(OrderTypeEnum.DOUBLE_ELEVEN2021)) {
            practiceGroupService.doubleEleven2021OrderCallback(order);
        }
    }

    @Override
    public Integer findOrderByGroupType(Integer userId, Integer sporadicId, String sporadic, DealStatusEnum status) {
        return studentPaymentOrderDao.findOrderByGroupType(userId, sporadicId, sporadic, status);
    }


    private void closeOrders(List<String> orderNoList) throws Exception {
        if (orderNoList.size() == 0) {
            return;
        }

        Calendar beforeTime = Calendar.getInstance();
        beforeTime.add(Calendar.MINUTE, -35);// 30分钟之前的时间
        Date beforeDate = beforeTime.getTime();

        List<StudentPaymentOrder> ordersOverTime = findOrdersOverTime(orderNoList, DealStatusEnum.ING, beforeDate);
        for (StudentPaymentOrder order : ordersOverTime) {
            try {
                order.setStatus(DealStatusEnum.FAILED);
                order.setMemo("超时未支付关闭");
                callOrderCallBack(order);
            } catch (Exception e) {
                e.printStackTrace();
                continue;
            }
        }

    }

    @Override
    public List<StudentPaymentOrderExportDto> ExportQueryPage(Map<String, Object> params) {
        return studentPaymentOrderDao.ExportQueryPage(params);
    }

    @Override
    public PageInfo<SporadicChargeInfoDto> sporadicQueryPage(SporadicOrderQueryInfo queryInfo) {
        PageInfo<SporadicChargeInfoDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<SporadicChargeInfoDto> dataList = null;
        int count = studentPaymentOrderDao.countSporadicPage(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentPaymentOrderDao.sporadicQueryPage(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public StudentPaymentOrder getOrderMoneyAmount(StudentPaymentOrderQueryInfo queryInfo) {
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);
        return studentPaymentOrderDao.getOrderMoneyAmount(params);
    }

    @Override
    public PageInfoOrder<StudentPaymentOrderDto> OrderQueryPage(StudentPaymentOrderQueryInfo queryInfo) {
        PageInfoOrder<StudentPaymentOrderDto> pageInfo = new PageInfoOrder<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentPaymentOrderDto> dataList = new ArrayList<>();
        int count = this.findCount(params);

        BigDecimal totalUserBalance = sysUserCashAccountDao.getTotalUserBalance(params);
        pageInfo.setTotalUserBalance(totalUserBalance);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentPaymentOrderDao.queryPageOrder(params);

            StudentPaymentOrder orderMoneyAmount = studentPaymentOrderDao.getOrderMoneyAmount(params);
            BigDecimal totalExpectAmount = orderMoneyAmount.getExpectAmount() != null ? orderMoneyAmount.getExpectAmount() : BigDecimal.ZERO;
            BigDecimal totalActualAmount = orderMoneyAmount.getActualAmount() != null ? orderMoneyAmount.getActualAmount() : BigDecimal.ZERO;
            BigDecimal totalBalancePaymentAmount = orderMoneyAmount.getBalancePaymentAmount() != null ? orderMoneyAmount.getBalancePaymentAmount() : BigDecimal.ZERO;

            params.put("chargeType", SporadicChargeTypeEnum.RECHARGE);
            BigDecimal totalChargeAmount = studentPaymentOrderDao.getChargeAmount(params);
            totalChargeAmount = totalChargeAmount != null ? totalChargeAmount : BigDecimal.ZERO;
            BigDecimal totalAdvanceAmount = totalChargeAmount.subtract(totalBalancePaymentAmount);

            pageInfo.setTotalAdvanceAmount(totalAdvanceAmount);
            pageInfo.setTotalExpectAmount(totalExpectAmount);
            pageInfo.setTotalActualAmount(totalActualAmount);
            pageInfo.setTotalRevenueAmount(totalActualAmount.subtract(totalAdvanceAmount));
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public Boolean getOrderStatus(Long id) throws Exception {
        StudentPaymentOrder studentPaymentOrder = studentPaymentOrderDao.get(id);
        List<StudentPaymentOrder> orderList = new ArrayList<>();
        orderList.add(studentPaymentOrder);
        if (studentPaymentOrder.getPaymentChannel().equals("ADAPAY")) {
            adaPayQuery(orderList);
        } else if (studentPaymentOrder.getPaymentChannel().equals("YQPAY")) {
            yqPayQuery(orderList);
        }
        return true;
    }

    @Override
    public Integer findOrganPayOrderNum(Integer organId) {
        return studentPaymentOrderDao.findOrganPayOrderNum(organId);
    }

    @Override
    public List<StudentPaymentOrder> getUserOrderByType(Integer userId, OrderTypeEnum type, DealStatusEnum status) {
        return studentPaymentOrderDao.getUserOrderByType(userId, type, status);
    }

    @Override
    public List<StudentPaymentOrder> reConfirmOrder(Date startTime, Date endTime) {
        if (startTime == null) {
            endTime = new Date();
            startTime = DateUtil.addMinutes(endTime, -20);
        }

        List<StudentPaymentOrder> orders = studentPaymentOrderDao.getAdaPayOrdersByTime(startTime, endTime);
        String tips = "";
        for (StudentPaymentOrder order : orders) {
            try {
                this.confirmOrder(order);
            } catch (Exception e) {
                tips += order.getOrderNo() + ":" + e.getMessage() + ",";
                continue;
            }
        }
        if (StringUtils.isNotBlank(tips)) {
            throw new BizException(tips);
        }
        return orders;
    }

    @Override
    public List<StudentPaymentOrderExportDto> getUserApplyOrders(Integer studentId, String musicGroupId) {
        List<StudentPaymentOrderExportDto> orders = studentPaymentOrderDao.getUserApplyOrders(studentId, musicGroupId);
        StudentRegistration studentRegister = studentRegistrationService.getStudentRegister(musicGroupId, studentId);
        for (StudentPaymentOrderExportDto order : orders) {
            if((DealStatusEnum.ING.equals(order.getStatus())||DealStatusEnum.SUCCESS.equals(order.getStatus())) &&
                studentRegister.getPayingStatus().equals(2)){
                order.setPayingStatus(2);
            }
            if (order.getOrderDetailList() == null) continue;
            for (StudentPaymentOrderDetail studentPaymentOrderDetail : order.getOrderDetailList()) {
                if (studentPaymentOrderDetail.getGoodsList() == null) continue;

                String childGoodIds = "";
                for (Goods goods : studentPaymentOrderDetail.getGoodsList()) {
                    if (StringUtils.isNotBlank(goods.getComplementGoodsIdList())) {
                        childGoodIds += StringUtils.isNotBlank(childGoodIds) ? "," : "" + goods.getComplementGoodsIdList();
                    }
                }
                if (StringUtils.isNotBlank(childGoodIds)) {
                    studentPaymentOrderDetail.setChildGoodsList(goodsDao.findGoodsByIds(childGoodIds));
                }
                if(studentPaymentOrderDetail.getType() == OrderDetailTypeEnum.CLOUD_TEACHER ||studentPaymentOrderDetail.getType() == OrderDetailTypeEnum.CLOUD_TEACHER_PLUS){
                	studentPaymentOrderDetail.setCloudTeacherOrderDto(cloudTeacherOrderService.queryOrderInfoByOrderId(studentPaymentOrderDetail.getPaymentOrderId()));
                }
            }
        }
        return orders;
    }

    @Override
    public StudentPaymentOrder getUserReplacementIngOrder(Integer userId, Integer replacementId) {
        return studentPaymentOrderDao.getUserReplacementIngOrder(userId, replacementId);
    }

	@Override
	public int batchUpdate(List<StudentPaymentOrder> studentPaymentOrderList) {
		return studentPaymentOrderDao.batchUpdate(studentPaymentOrderList);
	}

	@Override
	public PayStatus queryPayStatus(String paymentChannel, String orderNo, String transNo) throws Exception {
		if (StringUtils.isBlank(transNo)) {
			return PayStatus.FAILED;
		}
		if (StringUtils.equals(paymentChannel, "ADAPAY")) {
			Map<String, Object> payment = Payment.queryPayment(transNo);
			String status = (String) payment.get("status");
			if (status.equals("succeeded")) {
				return PayStatus.SUCCESSED;
			} else if (status.equals("failed")) {
				return PayStatus.FAILED;
			} else {
				return PayStatus.PAYING;
			}
		} else if (StringUtils.equals(paymentChannel, "YQPAY")) {
			String notifyUrl = ""; // 回调地址
			Map<String, Object> resultMap = new LinkedHashMap<>();
			resultMap.put("merOrderNoList", orderNo);
			Map<String, Object> requestMap = YqPayUtil.getRequestMap(notifyUrl, resultMap);

			RsqMsg rsqMsg = new RsqMsg(requestMap);

			Msg queryRs = yqPayFeignService.orderQuery(rsqMsg);

			if (queryRs.getCode().equals("88")) {
				// 更新订单状态
				String responseParameters = queryRs.getResponseParameters();
				List<Map<String, String>> responseList = JSON.parseObject(responseParameters, List.class);
				for (Map<String, String> response : responseList) {

					if (StringUtils.equals(response.get("tradeState"), "1")) {
						return PayStatus.SUCCESSED;
					} else if (StringUtils.equals(response.get("tradeState"), "0") || StringUtils.equals(response.get("tradeState"), "7")) {
						return PayStatus.FAILED;
					} else {
						return PayStatus.PAYING;
					}
				}
			}
		}
		throw new BizException("支付渠道错误");
	}

    @Override
    public Integer getOrderIdByMusical(String musicGroupId, Integer studentId) {
        return studentPaymentOrderDao.getOrderIdByMusical(musicGroupId,studentId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer activityGive(Integer vipGroupActivityId, StudentPaymentOrder studentPaymentOrder, Long vipGroupId, Long practiceGroupId, Integer teacherId) {
        Integer activityUserMapperId = null;
        if(vipGroupActivityId != null){
            ActivityUserMapper activityUserMapper = new ActivityUserMapper();
            activityUserMapper.setVipGroupId(vipGroupId);
            activityUserMapper.setPracticeGroupId(practiceGroupId);
            activityUserMapper.setPaymentOrderId(studentPaymentOrder.getId());
            activityUserMapper.setUserId(studentPaymentOrder.getUserId());
            activityUserMapper.setActivityId(vipGroupActivityId);
            activityUserMapper.setTeacherId(teacherId);
            activityUserMapper.setActualPrice(studentPaymentOrder.getExpectAmount());
            if(vipGroupId != null){
                activityUserMapper.setVipFlag(2);
            }else {
                activityUserMapper.setPracticeFlag(2);
            }
            //是否达到满赠标准
            VipGroupActivity activity = vipGroupActivityDao.get(vipGroupActivityId);
            if(activity.getFullMinusCourseTimes() == -1 || studentPaymentOrder.getActivityBuyNum() < activity.getFullMinusCourseTimes()){
                activityUserMapperDao.insert(activityUserMapper);
                return activityUserMapper.getId();
            }
            //是否赠送课程
            if(activity.getGiveCourseNum() > 0){
                if(activity.getGiveCourseType().equals("VIP")){
                    activityUserMapper.setGiveVipFlag(1);
                }else {
                    activityUserMapper.setGivePracticeFlag(1);
                }
            }
            //是否赠送会员(会员立即生效)
            if(activity.getGiveMemberTime() > 0){
                Date now = new Date();
                // 查询会员订单信息
                CloudTeacherOrder cloudTeacherOrder = new CloudTeacherOrder();
                cloudTeacherOrder.setStatus(2);
                cloudTeacherOrder.setUpdateTime(now);
                cloudTeacherOrder.setAmount(BigDecimal.ZERO);
                cloudTeacherOrder.setOrderId(studentPaymentOrder.getId());
                cloudTeacherOrder.setRemark("购买vip活动赠送会员");
                cloudTeacherOrder.setLevel(activity.getGiveMemberRankId());
                cloudTeacherOrder.setStudentId(studentPaymentOrder.getUserId());
                cloudTeacherOrder.setType(2);
                cloudTeacherOrder.setOrganId(studentPaymentOrder.getOrganId());
                PeriodEnum periodEnum = activity.getGivePeriodEnum();
                switch (periodEnum){
                    case DAY:
                        cloudTeacherOrder.setType(1);
                        break;
                    case MONTH:
                    case QUARTERLY:
                    case YEAR_HALF:
                        cloudTeacherOrder.setType(2);
                        break;
                    case YEAR:
                        cloudTeacherOrder.setType(3);
                        break;
                    default:
                        throw new BizException("不支持的周期类型");
                }
                Student student = studentService.get(studentPaymentOrder.getUserId());
                Date startDate = now;
                if (student.getMemberRankSettingId() == null || now.after(student.getMembershipEndTime())) {
                    cloudTeacherOrder.setStartTime(now);
                } else {
                    cloudTeacherOrder.setStartTime(DateUtil.addDays(student.getMembershipEndTime(), 1));
                    startDate = student.getMembershipEndTime();
                }
                switch (periodEnum){
                    case DAY:
                        cloudTeacherOrder.setEndTime(DateUtil.addDays(startDate, activity.getGiveMemberTime()));
                        cloudTeacherOrder.setTime(activity.getGiveMemberTime());
                        break;
                    case MONTH:
                        cloudTeacherOrder.setEndTime(DateUtil.addMonths(startDate, activity.getGiveMemberTime()));
                        cloudTeacherOrder.setTime(activity.getGiveMemberTime());
                        break;
                    case QUARTERLY:
                        cloudTeacherOrder.setEndTime(DateUtil.addMonths(startDate, activity.getGiveMemberTime() * 3));
                        cloudTeacherOrder.setTime(activity.getGiveMemberTime() * 3);
                        break;
                    case YEAR_HALF:
                        cloudTeacherOrder.setEndTime(DateUtil.addMonths(startDate, activity.getGiveMemberTime() * 6));
                        cloudTeacherOrder.setTime(activity.getGiveMemberTime() * 6);
                        break;
                    case YEAR:
                        cloudTeacherOrder.setEndTime(DateUtil.addYears(startDate, activity.getGiveMemberTime()));
                        cloudTeacherOrder.setTime(activity.getGiveMemberTime());
                        break;
                    default:
                        throw new BizException("不支持的周期类型");
                }
                cloudTeacherOrderService.insert(cloudTeacherOrder);
                // 添加会员有效时长
                studentService.updateMemberRank(cloudTeacherOrder, periodEnum);
                activityUserMapper.setGiveMemberFlag(2);
                activityUserMapper.setGiveMemberOrderId(cloudTeacherOrder.getId());
            }
            activityUserMapperDao.insert(activityUserMapper);
            activityUserMapperId = activityUserMapper.getId();
        }
        return activityUserMapperId;
    }
}
