package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.StudentPaymentOrderDao;
import com.ym.mec.biz.dal.dao.SysCouponCodeDao;
import com.ym.mec.biz.dal.dao.SysCouponDao;
import com.ym.mec.biz.dal.dto.HorseRaceLampDto;
import com.ym.mec.biz.dal.dto.SysCouponCodeDto;
import com.ym.mec.biz.dal.entity.StudentPaymentOrder;
import com.ym.mec.biz.dal.entity.SysCoupon;
import com.ym.mec.biz.dal.entity.SysCouponCode;
import com.ym.mec.biz.dal.enums.CouponTypeEnum;
import com.ym.mec.biz.dal.page.SysCouponCodeQueryInfo;
import com.ym.mec.biz.service.SysCouponCodeService;
import com.ym.mec.biz.service.SysCouponService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

@Service
public class SysCouponCodeServiceImpl extends BaseServiceImpl<Long, SysCouponCode>  implements SysCouponCodeService {
	
	@Autowired
	private SysCouponCodeDao sysCouponCodeDao;
	@Autowired
	private SysCouponService sysCouponService;
	@Autowired
	private SysCouponDao sysCouponDao;
	@Autowired
	private IdGeneratorService idGeneratorService;
	@Autowired
	private StudentPaymentOrderDao studentPaymentOrderDao;

	@Override
	public BaseDAO<Long, SysCouponCode> getDAO() {
		return sysCouponCodeDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
	public boolean exchangeCouponTest(Integer userId, Integer couponId, Long paymentOrderId, Integer exchangeNum) {
		return exchangeCoupon(userId, couponId, paymentOrderId, exchangeNum);
	}

	@Override
	@Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
	public boolean exchangeCoupon(Integer userId, Integer couponId, Long paymentOrderId, Integer exchangeNum) {
		if(Objects.isNull(userId)||Objects.isNull(couponId)||Objects.isNull(paymentOrderId)||Objects.isNull(exchangeNum)){
			throw new BizException("领取失败");
		}
		if(exchangeNum<=0){
			return true;
		}
		SysCoupon sysCoupon = sysCouponDao.lockCoupon(couponId);
		Date now = new Date();
		//如果优惠券不存在，或者已停用，或者不在领取有效期内，或者已消耗完，则此优惠券无效
		boolean invalid = Objects.isNull(sysCoupon)||0==sysCoupon.getStatus()||now.compareTo(sysCoupon.getStartDate())<0||now.compareTo(sysCoupon.getEndDate())>0;
		invalid = invalid || sysCoupon.getStockCount()!=-1&&sysCoupon.getConsumeNum()>=sysCoupon.getStockCount()&&(sysCoupon.getStockCount()-sysCoupon.getConsumeNum()<exchangeNum);
		if(invalid){
			throw new BizException("无效优惠券");
		}
		StudentPaymentOrder studentPaymentOrder = studentPaymentOrderDao.get(paymentOrderId);
		if(Objects.isNull(studentPaymentOrder)){
			throw new BizException("无效订单");
		}
		if(sysCoupon.getLimitExchangeNum()!=-1){
			int userUserNum = sysCouponCodeDao.countWithUserAndCoupon(userId, couponId);
			invalid = (userUserNum + exchangeNum)>sysCoupon.getLimitExchangeNum();
		}
		//如果用户使用数量超过优惠券限制则领取无效
		if(invalid){
			throw new BizException("当前优惠券最多可领取{}次", sysCoupon.getLimitExchangeNum());
		}
		SysCouponCode sysCouponCode = new SysCouponCode();
		sysCouponCode.setUserId(userId);
		sysCouponCode.setCouponId(couponId);
		sysCouponCode.setPaymentOrderId(paymentOrderId);
		sysCouponCode.setUsageStatus(0);
		switch (sysCoupon.getEffectiveType()){
			case DAYS:
				sysCouponCode.setUseStartDate(now);
				sysCouponCode.setUseDeadlineDate(DateUtil.addDays(now, sysCoupon.getDeadline()));
				break;
			case TIME_BUCKET:
				sysCouponCode.setUseStartDate(sysCoupon.getEffectiveStartTime());
				sysCouponCode.setUseDeadlineDate(sysCoupon.getEffectiveExpireTime());
				break;
			default:
				throw new BizException("无效优惠券", sysCoupon.getLimitExchangeNum());
		}
		List<SysCouponCode> couponCodes = new ArrayList<>();
		for (int i = 0; i < exchangeNum; i++) {
			SysCouponCode couponCode = new SysCouponCode();
			BeanUtils.copyProperties(sysCouponCode, couponCode);
			couponCode.setCode(String.valueOf(idGeneratorService.generatorId("coupon")));
			couponCodes.add(couponCode);
		}
		sysCouponCodeDao.batchInsert(couponCodes);
		sysCoupon.setConsumeNum(new AtomicInteger(sysCoupon.getConsumeNum()).addAndGet(exchangeNum));
		if(sysCoupon.getWarningStatus()==0&&sysCoupon.getStockCount()!=-1&&sysCoupon.getStockCount()-sysCoupon.getConsumeNum()<sysCoupon.getWarningStockNum()){
			sysCouponService.stockWarning(sysCoupon.getId(), sysCoupon.getName());
			sysCoupon.setWarningStatus(1);
		}
		sysCouponDao.update(sysCoupon);
		return true;
	}

	@Override
	public PageInfo<SysCouponCodeDto> querySysCouponUseList(SysCouponCodeQueryInfo queryInfo) {
		PageInfo<SysCouponCodeDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<SysCouponCodeDto> dataList = new ArrayList<>();
		int count = sysCouponCodeDao.countSysCouponUseList(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = sysCouponCodeDao.querySysCouponUseList(params);
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

    @Override
    public PageInfo<HorseRaceLampDto> queryHorseRaceLampDtoList(QueryInfo queryInfo) {
		PageInfo<HorseRaceLampDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<HorseRaceLampDto> dataList = new ArrayList<>();
		int count = sysCouponCodeDao.countHorseRaceLampDtoList(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = sysCouponCodeDao.queryHorseRaceLampDtoList(params);
		}
		pageInfo.setRows(dataList);
		return pageInfo;
    }

    @Override
    public List<SysCouponCode> queryCouponPage(BigDecimal amount, Integer userId) {

		return null;
    }

    @Override
	@Transactional(rollbackFor = Exception.class)
    public StudentPaymentOrder use(List<Integer> couponIdList, BigDecimal amount,Boolean useFlag) {
		StudentPaymentOrder studentPaymentOrder = new StudentPaymentOrder();
		if(couponIdList != null && couponIdList.size() > 0){
			Date date = new Date();
			List<SysCouponCodeDto> couponCodeDtoList = sysCouponCodeDao.findByIdList(couponIdList);
			if(couponIdList.size() != couponCodeDtoList.size()){
				throw new BizException("操作失败：优惠券数据异常");
			}
			BigDecimal fullAmount = BigDecimal.ZERO;
			BigDecimal faceAmount = BigDecimal.ZERO;

			for (SysCouponCodeDto sysCouponCodeDto : couponCodeDtoList) {
				if(sysCouponCodeDto.getUsageStatus() == 1){
					throw new BizException("操作失败：优惠券已使用");
				}
				if(sysCouponCodeDto.getUseStartDate().after(date)){
					throw new BizException("操作失败：不在优惠券使用期限");
				}
				if(date.after(sysCouponCodeDto.getUseDeadlineDate())){
					throw new BizException("操作失败：优惠券已过期");
				}
				CouponTypeEnum couponType = sysCouponCodeDto.getCouponType();
				switch (couponType){
					case DISCOUNT:
						throw new BizException("操作失败：暂不支持折扣劵");
					case FULL_REDUCTION:
						fullAmount = fullAmount.add(sysCouponCodeDto.getFullAmount());
						faceAmount = faceAmount.add(sysCouponCodeDto.getFaceValue());
						break;
					default:
						throw new BizException("操作失败：错误的优惠券类型");
				}
			}
			if(amount.compareTo(fullAmount) < 0){
				throw new BizException("操作失败：当前消费金额不满足优惠券满减条件");
			}
			amount = amount.subtract(faceAmount);
			if (amount.signum() < 0) {
				amount = BigDecimal.ZERO;
			}
			//使用优惠券
			if(useFlag){
				sysCouponCodeDao.use(couponIdList);
			}
			studentPaymentOrder.setCouponCodeId(StringUtils.join(couponIdList,","));
			studentPaymentOrder.setCouponRemitFee(faceAmount);
		}
		studentPaymentOrder.setExpectAmount(amount);
		studentPaymentOrder.setActualAmount(amount);
		return studentPaymentOrder;
    }

	@Override
	public List<SysCouponCodeDto> findByIdList(List<Integer> couponIdList) {
		return sysCouponCodeDao.findByIdList(couponIdList);
	}

	@Override
	public void quit(String couponCodeId) {
		if(StringUtils.isEmpty(couponCodeId)){
			return;
		}
		sysCouponCodeDao.quit(couponCodeId);
	}
}
