package com.ym.mec.biz.service.impl;


import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.SubjectDao;
import com.ym.mec.biz.dal.dao.SysMusicScoreAccompanimentDao;
import com.ym.mec.biz.dal.dao.SysMusicScoreCategoriesDao;
import com.ym.mec.biz.dal.dao.SysMusicScoreDao;
import com.ym.mec.biz.dal.dto.MusicScoreDto;
import com.ym.mec.biz.dal.entity.SysMusicScore;
import com.ym.mec.biz.dal.entity.SysMusicScoreAccompaniment;
import com.ym.mec.biz.dal.entity.SysMusicScoreCategories;
import com.ym.mec.biz.dal.page.SysExamSongQueryInfo;
import com.ym.mec.biz.service.SysMusicScoreService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.util.*;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.enums.ClientTypeEnum.SMART_PRACTICE;

@Service
public class SysMusicScoreServiceImpl extends BaseServiceImpl<Integer, SysMusicScore> implements SysMusicScoreService {
	
	@Autowired
	private SysMusicScoreDao sysMusicScoreDao;
	@Autowired
	private SysMusicScoreAccompanimentDao sysMusicScoreAccompanimentDao;
	@Autowired
	private SysUserFeignService sysUserFeignService;
	@Autowired
	private SysMusicScoreCategoriesDao sysMusicScoreCategoriesDao;
	@Autowired
	private SubjectDao subjectDao;

	@Override
	public BaseDAO<Integer, SysMusicScore> getDAO() {
		return sysMusicScoreDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateMusicScore(MusicScoreDto musicScoreDto) {
		SysMusicScore sysMusicScore = musicScoreDto.getSysMusicScore();
		sysMusicScoreDao.update(sysMusicScore);
		List<Integer> delExamSongAccompanimentIds = musicScoreDto.getDelExamSongAccompanimentIds();
		if(delExamSongAccompanimentIds != null && delExamSongAccompanimentIds.size() > 0){
			sysMusicScoreAccompanimentDao.batchDel(delExamSongAccompanimentIds);
		}
		List<SysMusicScoreAccompaniment> sysMusicScoreAccompaniments = musicScoreDto.getSysMusicScoreAccompaniments();
		if(sysMusicScoreAccompaniments != null && sysMusicScoreAccompaniments.size() > 0){
			List<Integer> subjectIdList = sysMusicScoreAccompaniments.stream().map(e -> e.getSubjectId()).collect(Collectors.toList());
			subjectIdList.removeAll(Collections.singleton(null));
			if(subjectIdList.size() > 0){
				List<Integer> idList = sysMusicScoreAccompaniments.stream().map(e -> e.getId()).collect(Collectors.toList());
				List<Integer> subjectIds = sysMusicScoreAccompanimentDao.findSubjectByMusicScoreId(sysMusicScore.getId(),idList);
				int subjectSize = subjectIds.size() + subjectIdList.size();
				subjectIdList.addAll(subjectIds);
				int size = new HashSet<>(subjectIdList).size();
				if(size < subjectSize){
					throw new BizException("操作失败：请勿提交重复的伴奏声部");
				}
			}
			//获取需要修改的伴奏
			List<SysMusicScoreAccompaniment> updateCollect = sysMusicScoreAccompaniments.stream().filter(e -> e.getId() != null).collect(Collectors.toList());
			if(updateCollect.size() > 0){
				sysMusicScoreAccompanimentDao.batchUpdate(updateCollect,sysMusicScore.getIsOpenMetronome());
			}
			//获取需要新增的伴奏
			List<SysMusicScoreAccompaniment> addCollect = sysMusicScoreAccompaniments.stream().filter(e -> e.getId() == null).collect(Collectors.toList());
			if(addCollect.size() > 0){
				sysMusicScoreAccompanimentDao.batchInsert(addCollect,sysMusicScore.getId(),sysMusicScore.getIsOpenMetronome());
			}
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void add(MusicScoreDto musicScoreDto) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if(sysUser == null){
			throw new BizException("请登录");
		}
		SysMusicScore sysMusicScore = musicScoreDto.getSysMusicScore();
		sysMusicScore.setCreateUserId(sysUser.getId());
		sysMusicScoreDao.insert(sysMusicScore);
		List<SysMusicScoreAccompaniment> sysMusicScoreAccompaniments = musicScoreDto.getSysMusicScoreAccompaniments();
		if(sysMusicScoreAccompaniments != null && sysMusicScoreAccompaniments.size() > 0){
			sysMusicScoreAccompanimentDao.batchInsert(sysMusicScoreAccompaniments,sysMusicScore.getId(),sysMusicScore.getIsOpenMetronome());
		}
		Integer categoriesId = sysMusicScore.getMusicScoreCategoriesId();
		if(categoriesId != null && sysMusicScore.getClientType() == SMART_PRACTICE){
			while (true){
				SysMusicScoreCategories categories = sysMusicScoreCategoriesDao.get(categoriesId);
				if(categories.getParentId() == 0){
					categories.setMusicScoreNum(categories.getMusicScoreNum() + 1);
					sysMusicScoreCategoriesDao.update(categories);
					return;
				}
				categoriesId = categories.getParentId();
			}
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void del(Integer id) {
		SysMusicScore sysMusicScore = sysMusicScoreDao.get(id);
		if(sysMusicScore == null){
			throw new BizException("曲目不存在");
		}
		sysMusicScoreDao.delete(id);
		sysMusicScoreAccompanimentDao.deleteBySongId(id);
		Integer categoriesId = sysMusicScore.getMusicScoreCategoriesId();
		if(categoriesId != null && sysMusicScore.getClientType() == SMART_PRACTICE){
			while (true){
				SysMusicScoreCategories categories = sysMusicScoreCategoriesDao.get(categoriesId);
				if(categories.getParentId() == 0){
					Integer musicScoreNum = categories.getMusicScoreNum() - 1;
					if(musicScoreNum < 0){
						musicScoreNum = 0;
					}
					categories.setMusicScoreNum(musicScoreNum);
					sysMusicScoreCategoriesDao.update(categories);
					return;
				}
				categoriesId = categories.getParentId();
			}
		}
	}

	private String getParentTreeName(Integer categoriesId){
		StringBuffer sb = new StringBuffer();
		while (true){
			SysMusicScoreCategories categories = sysMusicScoreCategoriesDao.get(categoriesId);
			if(sb.length() == 0){
				sb.append(categories.getName());
			}else {
				sb.insert(0,"/");
				sb.insert(0,categories.getName());
			}
			if(categories.getParentId() == 0){
				return sb.toString();
			}
			categoriesId = categories.getParentId();
		}
	}

	@Override
	public PageInfo<SysMusicScore> queryMusicScorePage(SysExamSongQueryInfo queryInfo) {
		PageInfo<SysMusicScore> sysMusicScorePageInfo = queryPage(queryInfo);
		List<SysMusicScore> rows = sysMusicScorePageInfo.getRows();
		if(rows != null && rows.size() > 0){
			Set<Integer> categoriesIds = rows.stream().map(e -> e.getMusicScoreCategoriesId()).collect(Collectors.toSet());
			//获取分部列表
			Map<Integer,String> organMap = MapUtil.convertMybatisMap(sysMusicScoreCategoriesDao.queryOrganByIds(categoriesIds));
			//获取声部列表
			Set<Integer> subjectIds = rows.stream().map(e -> e.getSubjectId()).collect(Collectors.toSet());
			subjectIds.removeAll(Collections.singleton(null));
			Map<Long, String> subjectMap = new HashMap();
			if(subjectIds.size() > 0){
				subjectMap = MapUtil.convertMybatisMap(subjectDao.findBySubjecIds(StringUtils.join(subjectIds, ",")));
			}
			Map<Integer,String> map = new HashMap<>(categoriesIds.size());
			for (Integer categoriesId : categoriesIds) {
				map.put(categoriesId,getParentTreeName(categoriesId));
			}
			for (SysMusicScore row : rows) {
				row.setCategoriesName(map.get(row.getMusicScoreCategoriesId()));
				row.setOrganName(organMap.get(row.getMusicScoreCategoriesId()));
				row.setSubjectName(subjectMap.get(row.getSubjectId()));
			}
		}
		return sysMusicScorePageInfo;
	}
}