package com.ym.mec.student.config;

import com.ym.mec.common.security.BaseAccessDeniedHandler;
import com.ym.mec.common.security.BaseAuthenticationEntryPoint;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableResourceServer;
import org.springframework.security.oauth2.config.annotation.web.configuration.ResourceServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configurers.ResourceServerSecurityConfigurer;

@Configuration
@EnableResourceServer
@EnableGlobalMethodSecurity(prePostEnabled = true)
public class ResourceServerConfig extends ResourceServerConfigurerAdapter {

    @Autowired
    private BaseAccessDeniedHandler baseAccessDeniedHandler;

    @Autowired
    private BaseAuthenticationEntryPoint baseAuthenticationEntryPoint;

    @Override
    public void configure(HttpSecurity http) throws Exception {
        http.csrf().disable().exceptionHandling().accessDeniedHandler(baseAccessDeniedHandler).authenticationEntryPoint(baseAuthenticationEntryPoint).and()
                .authorizeRequests().antMatchers("/wechat/*","/v2/api-docs", "/code/*", "/register/*", "/studentOrder/*", "/musicGroup/getSubjectGoodsAndInfo",
                "/musicGroup/getMusicGroupStatus", "/musicGroup/getOrderStatus","/sysCouponCode/queryPage",
                "/musicGroup/pay",
                "/musicGroup/rePay",
                "/studentManage/queryStudentPer",
                "/sporadicChargeInfo/get",
                "/sporadicChargeInfo/sporadicPay",
                "/repair/getRepairInfo",
                "/repair/payRepair",
                "/degree/*",
                "/practiceGroup/queryOrderInfo",
                "/systemDate/query",
                "/organization/getOrgans",
                "/repair/addGoodsSellOrder",
                "/repair/getStudentGoodsOrder",
                "/subjectChange/getChangeInfo",
                "/subjectChange/payChange",
                "/contracts/queryProduceContract",
                "/repair/studentPaymentGoodsOrder",
                "/studentCompetition/getWinnerList",
                "/musicGroup/getGradeList",
                "/studentCompetition/get","/musicGroup/preRegister",
                "/musicEnlightenmentQuestionnaire/addEnlightenmentQuestionnaire", "/musicEnlightenmentQuestionnaire/getUserMusicEnlightenmentQuestionnaire").permitAll().anyRequest().authenticated().and().httpBasic();
    }

    @Override
    public void configure(ResourceServerSecurityConfigurer resources) throws Exception {
        resources.authenticationEntryPoint(baseAuthenticationEntryPoint).accessDeniedHandler(baseAccessDeniedHandler);
    }

}
