package com.ym.mec.student.controller;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import com.ym.mec.biz.dal.entity.SysUserContracts;
import com.ym.mec.biz.dal.enums.CourseViewTypeEnum;
import com.ym.mec.biz.service.SysUserContractsService;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.enums.KitGroupPurchaseTypeEnum;
import com.ym.mec.biz.service.ContractService;
import com.ym.mec.common.controller.BaseController;

@RequestMapping("contracts")
@Api(tags = "协议服务")
@RestController
public class ContractsController extends BaseController {

	@Autowired
	private SysUserFeignService sysUserFeignService;

	@Autowired
	private ContractService contractService;

	@Autowired
	private SysUserContractsService sysUserContractsService;

	@ApiOperation("查询注册协议")
	@GetMapping(value = "queryRegisterContract")
	public Object queryRegisterContract() {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("获取用户信息失败");
		}
		return succeed(contractService.queryRegisterContract(sysUser.getId()));
	}

	@ApiOperation("查询商品协议")
	@GetMapping(value = "queryGoodsContract")
	public Object queryGoodsContract(String musicGroupId, String goodsIds, KitGroupPurchaseTypeEnum kitGroupPurchaseTypeEnum) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("获取用户信息失败");
		}
		return succeed(contractService.queryGoodsContract(sysUser.getId(), musicGroupId, goodsIds, kitGroupPurchaseTypeEnum));
	}

	@ApiOperation("查询乐团课程协议")
	@GetMapping(value = "queryMusicGroupCoursesContract")
	public Object queryMusicGroupCoursesContract(String musicGroupId) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("获取用户信息失败");
		}
		return succeed(contractService.queryMusicGroupCoursesContract(sysUser.getId(), musicGroupId));
	}

	@ApiOperation("查询VIP课程协议")
	@GetMapping(value = "queryVipGroupCoursesContract")
	public Object queryVipGroupCoursesContract(Long vipGroupId) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("获取用户信息失败");
		}
		return succeed(contractService.queryVipGroupCoursesContract(sysUser.getId(), vipGroupId));
	}

	@ApiOperation("查询陪练课课程协议")
	@GetMapping(value = "queryPracticeCoursesContract")
	public Object queryPracticeCoursesContract(int courseSectionNum, Date startDate, Date endDate, BigDecimal fee) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("获取用户信息失败");
		}
		return succeed(contractService.queryPracticeCoursesContract(sysUser.getId(), courseSectionNum, startDate, endDate, fee));
	}

	@ApiOperation("查询产品协议")
	@GetMapping(value = "queryProduceContract")
	public Object queryProduceContract(Integer userId, String musicGroupId, CourseViewTypeEnum courseViewType) {
		if(Objects.isNull(userId)){
			SysUser sysUser = sysUserFeignService.queryUserInfo();
			if (sysUser == null) {
				return failed("获取用户信息失败");
			}
			userId = sysUser.getId();
		}
		return succeed(contractService.queryProductContract(userId,musicGroupId, courseViewType));
	}

	@ApiOperation(value = "生成学生课程协议")
	@GetMapping("/transferProduceContractOnlyWithCourseList")
	public HttpResponseResult transferProduceContractOnlyWithCourseList(Integer userId){
		if(Objects.isNull(userId)){
			SysUser sysUser = sysUserFeignService.queryUserInfo();
			if (sysUser == null) {
				return failed("获取用户信息失败");
			}
			userId = sysUser.getId();
		}
		return contractService.transferProduceContractOnlyWithCourseList(userId);
	}

	@ApiOperation(value = "查询学生最新协议")
	@GetMapping("/getLatest")
	public HttpResponseResult<List<SysUserContracts>> getLatest(Integer userId) {
		if(Objects.isNull(userId)){
			SysUser sysUser = sysUserFeignService.queryUserInfo();
			if (sysUser == null) {
				return failed("获取用户信息失败");
			}
			userId = sysUser.getId();
		}
		List<SysUserContracts> sysUserContracts = sysUserContractsService.getUserAllContracts(userId);

		if(CollectionUtils.isEmpty(sysUserContracts)){
			return succeed(Collections.EMPTY_LIST);
		}

		Map<Integer, List<SysUserContracts>> versionContractMap = sysUserContracts.stream().collect(Collectors.groupingBy(SysUserContracts::getVersion));

		List<SysUserContracts> result = new ArrayList<>();
		for (Map.Entry<Integer, List<SysUserContracts>> versionContractMapEntry : versionContractMap.entrySet()) {
			versionContractMapEntry.getValue().sort(Comparator.comparing(SysUserContracts::getCreateTime).reversed());
			result.add(versionContractMapEntry.getValue().get(0));
		}

		return succeed(result);
	}
}
