package com.ym.mec.student.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.SysConfigDao;
import com.ym.mec.biz.dal.dto.MemberPayParamDto;
import com.ym.mec.biz.dal.entity.MemberRankSetting;
import com.ym.mec.biz.dal.enums.PeriodEnum;
import com.ym.mec.biz.dal.page.CloudTeacherOrderQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@RequestMapping("memberRank")
@Api(tags = "会员收费设置服务")
@RestController
public class MemberRankController extends BaseController {

	@Autowired
	private MemberFeeSettingService memberFeeSettingService;

	@Autowired
	private MemberRankSettingService memberRankSettingService;

	@Autowired
	private MemberRankPrivilegesService memberRankPrivilegesService;
	
	@Autowired
	private CloudTeacherOrderService cloudTeacherOrderService;

	@Autowired
	private SysUserFeignService sysUserFeignService;

	@Autowired
	private SysConfigDao sysConfigDao;

	@ApiOperation(value = "获取所有的收费标准")
	@GetMapping("/queryList")
	public Object getMemberDefaultFee() {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed(HttpStatus.FORBIDDEN, "请登录");
		}

		List<MemberRankSetting> result = memberRankSettingService.queryListByIsDefault(false);

		for (MemberRankSetting mrs : result) {
			mrs.setMemberFeeSetting(memberFeeSettingService.findByRankIdAndOrganId(sysUser.getOrganId(), mrs.getId()));
			mrs.setMemberPrivilegesItemList(memberRankPrivilegesService.queryByMemberRankId(mrs.getId()));
		}
		return succeed(result);
	}

	@ApiOperation(value = "购买会员")
	@PostMapping("/buy")
	public Object buy(@RequestBody MemberPayParamDto memberPayParamDto) throws Exception {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed(HttpStatus.FORBIDDEN, "请登录");
		}
		memberPayParamDto.setUserId(sysUser.getId());
		memberPayParamDto.setBuyNum(1);
		memberPayParamDto.setGiveNum(0);
		return succeed(memberRankSettingService.buy(memberPayParamDto));
	}

	@ApiOperation(value = "2021-09活动购买会员")
	@PostMapping("/activeBuy")
	public HttpResponseResult ActiveBuy(@RequestBody MemberPayParamDto memberPayParamDto) throws Exception {
		Date nowTime = new Date();
		String startTimeStr = sysConfigDao.findConfigValue("cloud_teacher_active_start_time");
		Date startTime = DateUtil.stringToDate(startTimeStr);
		if (startTime.compareTo(nowTime) > 0) {
			throw new BizException("活动还未开始，谢谢关注");
		}
		String endTimeStr = sysConfigDao.findConfigValue("cloud_teacher_active_end_time");
		Date endTime = DateUtil.stringToDate(endTimeStr);
		if (endTime.compareTo(nowTime) <= 0) {
			throw new BizException("活动已结束，谢谢关注");
		}
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed(HttpStatus.FORBIDDEN, "请登录");
		}
		if(sysUser.getOrganId() == 43){
			throw new BizException("活动仅对乐团老生学员开放");
		}
		//买2赠1、买3赠2
		Integer buyNum = memberPayParamDto.getBuyNum();
		int giveNum = buyNum == 2?1:buyNum == 3?2:0;
		String remark = buyNum == 2?"2赠1":buyNum == 3?"3赠2":buyNum == 1?"单月":"其他";
		return memberRankSettingService.ActiveBuy(sysUser.getId(),
				memberPayParamDto.getMemberRankId(),
				PeriodEnum.MONTH,
				memberPayParamDto.getAmount(),
				memberPayParamDto.isUseBalance(),
				buyNum,
				giveNum,
				202109,remark,memberPayParamDto.isRepay());
	}
	
    @ApiOperation(value = "查询订单列表")
    @GetMapping("/queryOrderPage")
    public Object queryOrderPage(CloudTeacherOrderQueryInfo queryInfo) {
    	SysUser user = sysUserFeignService.queryUserInfo();
		if (user == null) {
			return failed(HttpStatus.FORBIDDEN, "请登录");
		}
		queryInfo.setStudentId(user.getId());
        return succeed(cloudTeacherOrderService.queryPage(queryInfo));
    }

    @ApiOperation(value = "获取学员购买的云教练活动订单")
    @GetMapping("/queryActiveOrderPage")
    public Object queryActiveOrderPage() {
    	SysUser user = sysUserFeignService.queryUserInfo();
		if (user == null) {
			return failed(HttpStatus.FORBIDDEN, "请登录");
		}
        return succeed(cloudTeacherOrderService.queryActiveOrderPage(user.getId(),202109,null));
    }

}
