package com.ym.mec.student.controller;

import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentType.ADD_STUDENT;
import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentType.MUSIC_APPLY;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.MusicGroupPaymentCalenderDao;
import com.ym.mec.biz.dal.dao.MusicGroupPaymentCalenderDetailDao;
import com.ym.mec.biz.dal.dao.StudentPreRegistrationDao;
import com.ym.mec.biz.dal.dto.MusicGroupSubjectGoodsAndInfoDto;
import com.ym.mec.biz.dal.dto.RegisterPayDto;
import com.ym.mec.biz.dal.dto.RenewParamDto;
import com.ym.mec.biz.dal.entity.ApprovalStatus;
import com.ym.mec.biz.dal.entity.Goods;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentCalenderStatusEnum;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalenderDetail;
import com.ym.mec.biz.dal.entity.MusicGroupQuit;
import com.ym.mec.biz.dal.entity.MusicGroupStudentFee.PaymentStatus;
import com.ym.mec.biz.dal.entity.StudentPaymentOrder;
import com.ym.mec.biz.dal.entity.StudentPaymentOrderDetail;
import com.ym.mec.biz.dal.entity.StudentPreRegistration;
import com.ym.mec.biz.dal.entity.StudentRegistration;
import com.ym.mec.biz.dal.enums.CourseViewTypeEnum;
import com.ym.mec.biz.dal.enums.DealStatusEnum;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.enums.MusicGroupStatusEnum;
import com.ym.mec.biz.dal.enums.OrderDetailTypeEnum;
import com.ym.mec.biz.dal.enums.OrderTypeEnum;
import com.ym.mec.biz.dal.enums.PayStatus;
import com.ym.mec.biz.dal.enums.PaymentStatusEnum;
import com.ym.mec.biz.service.MusicGroupService;
import com.ym.mec.biz.service.MusicGroupSubjectPlanService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.biz.service.StudentPaymentOrderDetailService;
import com.ym.mec.biz.service.StudentPaymentOrderService;
import com.ym.mec.biz.service.StudentRegistrationService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;

@RequestMapping("musicGroup")
@Api(tags = "乐团服务")
@RestController
public class MusicGroupController extends BaseController {

    @Resource
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private MusicGroupService musicGroupService;
    @Autowired
    private MusicGroupSubjectPlanService musicGroupSubjectPlanService;
    @Autowired
    private StudentRegistrationService studentRegistrationService;
    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;
    @Autowired
    private StudentPaymentOrderDetailService studentPaymentOrderDetailService;
    @Autowired
    private MusicGroupPaymentCalenderDetailDao musicGroupPaymentCalenderDetailDao;
    @Autowired
    private MusicGroupPaymentCalenderDao musicGroupPaymentCalenderDao;
    @Autowired
    private StudentPreRegistrationDao studentPreRegistrationDao;
    @Autowired
    private OrganizationService organizationService;

    @ApiOperation("预报名")
    @PostMapping(value = "/preRegister")
    public Object preRegister(@RequestBody StudentPreRegistration studentPreRegistration) {
        return musicGroupService.preRegister(studentPreRegistration) ? succeed() : failed();
    }

    @ApiOperation("预报名信息查询")
    @GetMapping(value = "/queryPreRegister")
    public Object queryPreRegister(String musicGroupId) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }

        return succeed(studentPreRegistrationDao.queryByMusicGroupIdAndUserId(sysUser.getId(), musicGroupId));
    }

    @ApiOperation("获取学生所在乐团列表")
    @GetMapping(value = "/queryUserMusicGroups")
    public Object queryUserMusicGroups() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return succeed(musicGroupService.queryUserMusicGroups(sysUser.getId()));
    }

    @ApiOperation("获取学生所在乐团列表")
    @GetMapping(value = "/queryStudentMusicGroupInfo")
    public Object queryStudentMusicGroupInfo(String musicGroupId) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return succeed(studentRegistrationService.queryStudentMusicGroupInfo(sysUser.getId(),musicGroupId));
    }

    @ApiOperation("获取学生个人中心乐团信息")
    @GetMapping(value = "/queryPersonalMusicGroups")
    public Object queryPersonalMusicGroups() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return succeed(musicGroupService.queryPersonalMusicGroups(sysUser.getId()));
    }

    @ApiOperation(value = "根据学员编号，获取有关联的老师列表，获取学员通讯录列表")
    @GetMapping("/findTeachersByStuId")
    public Object findTeachersByStuId(String search) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return succeed(musicGroupService.findTeachersByStuId(sysUser.getId(), search));
    }


    @ApiOperation(value = "获取乐团声部费用信息及乐器和辅件")
    @GetMapping("/getSubjectGoodsAndInfo")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "subjectId", value = "声部编号", required = true, dataType = "Integer")})
    public HttpResponseResult<MusicGroupSubjectGoodsAndInfoDto> getSubjectGoodsAndInfo(String musicGroupId, Integer subjectId) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        return succeed(musicGroupSubjectPlanService.getSubjectGoodsAndInfo(musicGroupId, subjectId));
    }

    @ApiOperation(value = "乐团报名缴费状态查询")
    @GetMapping("/getMusicGroupStatus")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团id", required = true, dataType = "String"),
            @ApiImplicitParam(name = "userId", value = "用户id", required = true, dataType = "int")
    })
    public HttpResponseResult getMusicGroupStatus(String musicGroupId, Integer userId) throws Exception {
        MusicGroup musicGroup = musicGroupService.get(musicGroupId);
        if (musicGroup == null) {
            return failed("乐团不存在");
        }
        SysUser sysUser = sysUserFeignService.queryUserInfo();

        if (sysUser != null && Objects.nonNull(sysUser.getId())) {
            userId = sysUser.getId();
        }

        StudentRegistration studentRegistration = studentRegistrationService.queryByUserIdAndMusicGroupId(userId, musicGroupId);
        if (studentRegistration == null) {
            return failed(HttpStatus.CONTINUE, "乐团报名信息不存在");
        }
        
        // 判断是否可以继续购买
        List<StudentPaymentOrderDetail> studentPaymentOrderDetailList = studentPaymentOrderDetailService.findUserApplyOrder(userId, musicGroupId, DealStatusEnum.SUCCESS);
        if(studentPaymentOrderDetailList != null && studentPaymentOrderDetailList.size() > 0){
        	List<OrderDetailTypeEnum> typeList = studentPaymentOrderDetailList.stream().map(t -> t.getType()).collect(Collectors.toList());
        	//判断是否所有类型都购买完
			if (musicGroup.getCourseViewType() == CourseViewTypeEnum.MEMBER_FEE) {
				if (typeList.contains(OrderDetailTypeEnum.MUSICAL) && typeList.contains(OrderDetailTypeEnum.ACCESSORIES)
						&& (typeList.contains(OrderDetailTypeEnum.CLOUD_TEACHER) || typeList.contains(OrderDetailTypeEnum.CLOUD_TEACHER_PLUS))) {
					return failed(HttpStatus.PROCESSING, "您已缴费,请等待乐团开启");
				}
			} else {
				if (typeList.contains(OrderDetailTypeEnum.MUSICAL)
						&& typeList.contains(OrderDetailTypeEnum.ACCESSORIES)
						&& (typeList.contains(OrderDetailTypeEnum.CLOUD_TEACHER) || typeList.contains(OrderDetailTypeEnum.CLOUD_TEACHER_PLUS) || typeList.contains(OrderDetailTypeEnum.COURSE)
								|| typeList.contains(OrderDetailTypeEnum.HIGH_ONLINE_COURSE) || typeList.contains(OrderDetailTypeEnum.SINGLE)
								|| typeList.contains(OrderDetailTypeEnum.MIX) || typeList.contains(OrderDetailTypeEnum.HIGH)
								|| typeList.contains(OrderDetailTypeEnum.VIP) || typeList.contains(OrderDetailTypeEnum.DEMO)
								|| typeList.contains(OrderDetailTypeEnum.COMPREHENSIVE) || typeList.contains(OrderDetailTypeEnum.ENLIGHTENMENT)
								|| typeList.contains(OrderDetailTypeEnum.TRAINING_SINGLE) || typeList.contains(OrderDetailTypeEnum.TRAINING_MIX)
								|| typeList.contains(OrderDetailTypeEnum.HIGH_ONLINE) || typeList.contains(OrderDetailTypeEnum.MUSIC_NETWORK) || typeList
									.contains(OrderDetailTypeEnum.CLASSROOM))) {
					return failed(HttpStatus.PROCESSING, "您已缴费,请等待乐团开启");
				}
			}
        }
        
        /*if (studentRegistration.getPaymentStatus() != null && studentRegistration.getPaymentStatus() == PaymentStatusEnum.YES) {
            
        }*/

        if (studentRegistration.getPaymentStatus() == PaymentStatusEnum.NO) {
            return failed("乐团还未开启缴费，请等待通知");
        }
        if (musicGroup.getStatus() != MusicGroupStatusEnum.APPLY &&
                musicGroup.getStatus() != MusicGroupStatusEnum.PAY &&
                musicGroup.getStatus() != MusicGroupStatusEnum.PREPARE &&
                musicGroup.getStatus() != MusicGroupStatusEnum.PROGRESS
        ) {
            return failed("乐团在" + musicGroup.getStatus().getMsg() + ",不能缴费");
        }
        String err = studentRegistrationService.checkRegOrPayStatus(musicGroupId);
        if (err != null) {
            return failed(err + "不能缴费");
        }
        studentRegistration.setOwnershipType(musicGroup.getOwnershipType());
        studentRegistration.setChargeTypeId(musicGroup.getChargeTypeId());
        if (sysUser != null) {
            studentRegistration.setGender(sysUser.getGender());
        }

        return succeed(studentRegistration);
    }

    @ApiOperation(value = "续费")
    @PostMapping("/renew")
    public HttpResponseResult renew(@RequestBody RenewParamDto renewParamDto) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        Integer userId = sysUser.getId();
        Long calenderId = renewParamDto.getCalenderId();
        if(calenderId == null){
            throw new BizException("参数校验失败");
        }
        MusicGroupPaymentCalender calender = musicGroupPaymentCalenderDao.get(calenderId);
        if (renewParamDto.getRepeatPay() == false) {
            MusicGroupPaymentCalender.PaymentType paymentType = calender.getPaymentType();
            OrderTypeEnum type = paymentType== ADD_STUDENT?OrderTypeEnum.ADD_STUDENT:paymentType == MUSIC_APPLY?OrderTypeEnum.APPLY:OrderTypeEnum.RENEW;
            // 判断是否存在支付中的记录
            List<StudentPaymentOrder> list = studentPaymentOrderService.queryByCondition(GroupType.MUSIC, calender.getMusicGroupId(), userId, DealStatusEnum.ING,
                    type);
            if (list != null && list.size() > 0) {
                for (StudentPaymentOrder order : list) {
                    // 查询订单状态
                    PayStatus payStatus = studentPaymentOrderService.queryPayStatus(order.getPaymentChannel(), order.getOrderNo(), order.getTransNo());
                    if(payStatus == PayStatus.SUCCESSED){
                        throw new BizException("订单已支付成功，请勿重复支付");
                    }/*else if(payStatus == PayStatus.PAYING){
                        throw new BizException("订单还在交易中，请稍后重试");
                    }*/
                    return failed(HttpStatus.CONTINUE, "您有待支付的订单");
                }
            }
        }
        MusicGroupPaymentCalenderDetail calenderDetail = musicGroupPaymentCalenderDetailDao.findByCalenderIdAndUserId(calenderId, userId);
        if (calenderDetail == null) {
            throw new BizException("缴费项不存在该学员，请联系教务老师");
        }
        if (calenderDetail.getPaymentStatus() == PaymentStatus.PAID_COMPLETED) {
            throw new BizException("您已缴费，请勿重复提交");
        }
        //缴费项目已开启或者单独开启
        Map renew;
        if (calender.getStatus() == PaymentCalenderStatusEnum.OPEN || calenderDetail.getOpen() == 1) {
            renewParamDto.setUserId(userId);
            renew = musicGroupService.renew(renewParamDto);
        } else if (calender.getStatus() == PaymentCalenderStatusEnum.OVER) {
            throw new BizException("缴费已截止,如有问题请联系指导老师");
        } else {
            throw new BizException("缴费暂未开始,如有问题请联系指导老师");
        }
        if (renew.containsKey("tradeState")) {
            return failed(HttpStatus.CREATED, renew, "恭喜您,缴费成功!");
        }
        return succeed(renew);
    }

    @ApiOperation(value = "申请退团")
    @PostMapping("/quitMusicGroup")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团id", required = true, dataType = "String"),
            @ApiImplicitParam(name = "reason", value = "原因", required = true, dataType = "String")})
    public HttpResponseResult quitMusicGroup(String musicGroupId, String reason) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed(HttpStatus.FORBIDDEN, "请登录");
		}
		MusicGroupQuit musicGroupQuit = new MusicGroupQuit();
		musicGroupQuit.setUserId(sysUser.getId());
		musicGroupQuit.setMusicGroupId(musicGroupId);
		musicGroupQuit.setUserComment(reason);
		musicGroupQuit.setStatus(ApprovalStatus.PROCESSING);
        return succeed(musicGroupService.applyQuitMusicGroup(null, musicGroupQuit, sysUser.getId()));
    }

    @ApiOperation(value = "取消退团申请")
    @PostMapping("/cancelQuitMusicGroup")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团id", required = true, dataType = "String"),
            @ApiImplicitParam(name = "reason", value = "取消原因", required = true, dataType = "String")})
    public HttpResponseResult cancelQuitMusicGroup(String musicGroupId, String reason) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        Integer userId = sysUser.getId();

        return succeed(musicGroupService.cancelQuitMusicGroup(userId, musicGroupId, reason));
    }

    @ApiOperation(value = "查询是否有待支付的报名订单")
    @GetMapping("/checkWaitPayApplyOrder")
    @ApiImplicitParams({@ApiImplicitParam(name = "registerId", value = "注册id", required = true, dataType = "int")})
    public HttpResponseResult<StudentPaymentOrder> checkWaitPayApplyOrder(Long registerId) throws Exception {

        StudentRegistration studentRegistration = studentRegistrationService.get(registerId);
        if (studentRegistration == null) {
            return failed("报名信息有误，请核查");
        }
        Integer userId = studentRegistration.getUserId();

        //判断用户是否已存在订单
        List<StudentPaymentOrder> applyOrderList = studentPaymentOrderService.findMusicGroupApplyOrderByStatus(userId, studentRegistration.getMusicGroupId(), DealStatusEnum.ING);
        
        if(applyOrderList != null && applyOrderList.size() > 0){
        	return succeed(applyOrderList.get(0));
        }
        return succeed(null);
    }

    @ApiOperation(value = "乐团报名支付")
    @PostMapping("/pay")
    public HttpResponseResult pay(@RequestBody RegisterPayDto registerPayDto) throws Exception {

        StudentRegistration studentRegistration = studentRegistrationService.get(registerPayDto.getRegisterId().longValue());
        if (studentRegistration == null) {
            return failed("报名信息有误，请核查");
        }

        String err = studentRegistrationService.checkRegOrPayStatus(studentRegistration.getMusicGroupId());
        if (err != null) {
            return failed(err + "不可缴费，请联系教务老师");
        }

        Integer userId = studentRegistration.getUserId();

        //判断用户是否已存在订单
        List<StudentPaymentOrder> applyOrderList = studentPaymentOrderService.findMusicGroupApplyOrderByStatus(userId, studentRegistration.getMusicGroupId(), DealStatusEnum.ING);
        
        if (applyOrderList != null && applyOrderList.size() > 0) {
        	StudentPaymentOrder applyOrder = applyOrderList.get(0);
        	// 查询订单状态
            PayStatus payStatus = studentPaymentOrderService.queryPayStatus(applyOrder.getPaymentChannel(), applyOrder.getOrderNo(), applyOrder.getTransNo());
            if(payStatus == PayStatus.SUCCESSED){
        		throw new BizException("订单已支付成功，请勿重复支付");
        	}/*else if(payStatus == PayStatus.PAYING){
        		throw new BizException("订单还在交易中，请稍后重试");
        	}*/
            return failed(HttpStatus.CONTINUE, "您有待支付的订单");
        }
        Map payMap = musicGroupService.pay(registerPayDto);
        if (payMap.containsKey("hasPaidZero")) {
            return failed(HttpStatus.RESET_CONTENT, payMap, "报名审核中");
        }
        if (payMap.containsKey("tradeState")) {
            return failed(HttpStatus.CREATED, payMap, "恭喜您,报名成功!");
        }
        return succeed(payMap);
    }

    @ApiOperation(value = "乐团报名重新支付")
    @PostMapping("/rePay")
    @ApiImplicitParams({@ApiImplicitParam(name = "registerPayDto", value = "支付信息", required = true, dataType = "RegisterPayDto")})
    public HttpResponseResult rePay(@RequestBody RegisterPayDto registerPayDto) throws Exception {
        StudentRegistration studentRegistration = studentRegistrationService.get(registerPayDto.getRegisterId().longValue());
        if (studentRegistration == null) {
            return failed("报名信息有误，请核查");
        }

        String err = studentRegistrationService.checkRegOrPayStatus(studentRegistration.getMusicGroupId());
        if (err != null) {
            return failed(err + "不可缴费，请联系教务老师");
        }
//        Integer userId = studentRegistration.getUserId();
        /*StudentPaymentOrder ApplyOrder = studentPaymentOrderService.findMusicGroupApplyOrderByStatus(userId, studentRegistration.getMusicGroupId(), DealStatusEnum.SUCCESS);
        if (ApplyOrder != null) {
            return failed("您已支付成功，请勿重复支付");
        }*/
        Map payMap = musicGroupService.rePay(registerPayDto);
        if (payMap.containsKey("hasPaidZero")) {
            return failed(HttpStatus.RESET_CONTENT, payMap, "报名审核中");
        }
        if (payMap.containsKey("tradeState")) {
            return failed(HttpStatus.CREATED, payMap, "恭喜您,报名成功!");
        }
        return succeed(payMap);
    }

    @ApiOperation(value = "订单状态查询")
    @GetMapping("/getOrderStatus")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderNo", value = "订单号", required = true, dataType = "String")})
    public HttpResponseResult getOrderStatus(String orderNo) {
        if (StringUtils.isBlank(orderNo)) {
            return failed("请指定订单");
        }
        StudentPaymentOrder orderByOrderNo = studentPaymentOrderService.findOrderByOrderNo(orderNo);
        if (Objects.isNull(orderByOrderNo)) {
            return failed("未找到指定订单");
        }
        MusicGroup musicGroup = musicGroupService.get(orderByOrderNo.getMusicGroupId());
        List<Goods> goodsList = studentPaymentOrderDetailService.findApplyOrderGoods(orderByOrderNo.getId());
        HashMap<String, Object> orderDetail = new HashMap<>();
        orderDetail.put("order", orderByOrderNo);
        orderDetail.put("goods", goodsList);
        orderDetail.put("course", musicGroup.getCourseForm());

        return succeed(orderDetail);
    }

    @ApiOperation(value = "获取乐团的年级列表")
    @GetMapping("/getGradeList")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团编号", required = true, dataType = "String")})
    public HttpResponseResult<Map<Integer, String>> getGradeList(String musicGroupId) {
        MusicGroup musicGroup = musicGroupService.get(musicGroupId);
        return succeed(organizationService.getGradeList(musicGroup.getOrganId()));
    }
}
