package com.ym.mec.teacher.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.PracticeGroupSellPriceDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dto.PracticeGroupApplyBaseInfoDto;
import com.ym.mec.biz.dal.dto.PracticeGroupApplyDto;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.Teacher;
import com.ym.mec.biz.service.PracticeGroupService;
import com.ym.mec.biz.service.VipGroupService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Map;
import java.util.Objects;

import static com.ym.mec.biz.dal.enums.PracticeGroupType.CHARGE;

@Api(tags = "网管课-教师端")
@RequestMapping("teacherPracticeGroup")
@RestController
public class TeacherPracticeGroupController extends BaseController {

	@Autowired
	private PracticeGroupService practiceGroupService;
	@Autowired
	private SysUserFeignService sysUserFeignService;
	@Autowired
	private TeacherDao teacherDao;
	@Autowired
	private PracticeGroupSellPriceDao practiceGroupSellPriceDao;

	@ApiOperation("获取网管课分部价格")
	@RequestMapping("/queryPracticeGroupSellPrice")
	public Object queryPracticeGroupSellPrice(){
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("用户信息获取失败");
		}
		Teacher teacher = teacherDao.get(sysUser.getId());
		if(Objects.isNull(teacher)){
			return failed("请指定指导老师!");
		}
		return succeed(practiceGroupSellPriceDao.get(teacher.getTeacherOrganId()));
	}

	@ApiOperation("课酬总费用")
	@GetMapping("/getPracticeGroupCostCount")
	public Object getPracticeGroupCostCount(){
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("用户信息获取失败");
		}
		Teacher teacher = teacherDao.get(sysUser.getId());
		PracticeGroupApplyBaseInfoDto practice = new PracticeGroupApplyBaseInfoDto();
		practice.setUserId(sysUser.getId());
		practice.setOrganId(teacher.getOrganId());
		Map results = practiceGroupService.countPracticeGroupPredictFee(practice,practice.getUserId());
		return succeed(results);
	}

	@ApiOperation(value = "教师端网管课活动排课")
	@PostMapping("/createPracticeGroup")
	public Object createPracticeGroup(@RequestBody PracticeGroupApplyDto practiceGroupApplyDto){
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("用户信息获取失败");
		}
		PracticeGroupApplyBaseInfoDto applyBaseInfo = practiceGroupApplyDto.getPracticeGroupApplyBaseInfoDto();
		Integer userId = sysUser.getId();
		applyBaseInfo.setUserId(userId);
		Teacher teacher = teacherDao.get(userId);
		if(Objects.isNull(teacher)){
			return failed("请指定指导老师!");
		}
		SysUser student = teacherDao.getUser(applyBaseInfo.getStudentId());
		if(Objects.isNull(student)||Objects.isNull(student.getOrganId())){
			return failed("学员信息异常");
		}
		applyBaseInfo.setOrganId(student.getOrganId());
		for (CourseSchedule courseSchedule : practiceGroupApplyDto.getCourseSchedules()) {
			courseSchedule.setEndClassTime(DateUtil.addMinutes(courseSchedule.getStartClassTime(), applyBaseInfo.getSingleClassMinutes()));
		}
		return practiceGroupService.createActivityPracticeGroup(practiceGroupApplyDto);
	}


	@ApiOperation(value = "网管课申请")
	@PostMapping("/practiceGroupApply")
	public Object practiceGroupApply(@RequestBody PracticeGroupApplyDto practiceGroupApplyDto){
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("用户信息获取失败");
		}
		PracticeGroupApplyBaseInfoDto applyBaseInfoDto = practiceGroupApplyDto.getPracticeGroupApplyBaseInfoDto();
		if(Objects.isNull(applyBaseInfoDto.getOrganId())){
			SysUser student = teacherDao.getUser(applyBaseInfoDto.getStudentId());
			if(Objects.isNull(student)||Objects.isNull(student.getOrganId())){
				return failed("学员信息异常");
			}
			applyBaseInfoDto.setOrganId(student.getOrganId());
		}
		applyBaseInfoDto.setType(CHARGE);
		applyBaseInfoDto.setMemo("指导老师老师端申请网管课");
		applyBaseInfoDto.setUserId(sysUser.getId());
		return practiceGroupService.createPracticeGroup(practiceGroupApplyDto);
	}

}
