package com.ym.mec.web.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.dto.ActivityPlanAddDto;
import com.ym.mec.biz.dal.entity.ActivityPlan;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.page.ActivityPlanQueryInfo;
import com.ym.mec.biz.service.ActivityPlanService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.exception.BizException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

@Api(tags = "活动方案")
@RequestMapping("activityPlan")
@RestController
public class ActivityPlanController extends BaseController {

    @Autowired
    private ActivityPlanService activityPlanService;
    @Autowired
    private SysUserFeignService sysUserFeignService;
	@Autowired
	private EmployeeDao employeeDao;

    @ApiOperation(value = "新增活动方案")
    @PostMapping("/addVipGroupActivity")
    @PreAuthorize("@pcs.hasPermissions('activityPlan/addActivityPlan')")
    public Object addActivityPlan(@RequestBody ActivityPlanAddDto activityPlanAddDto){
        activityPlanService.addActivityPlan(activityPlanAddDto);
        return succeed();
    }

    @ApiOperation(value = "修改活动方案")
    @PostMapping("/updateActivityPlan")
    @PreAuthorize("@pcs.hasPermissions('activityPlan/updateActivityPlan')")
    public Object updateActivityPlan(@RequestBody ActivityPlanAddDto activityPlanAddDto){
        activityPlanService.updateActivityPlan(activityPlanAddDto);
        return succeed();
    }

    @ApiOperation(value = "修改活动方案")
    @PostMapping("/enable")
    @PreAuthorize("@pcs.hasPermissions('activityPlan/enable')")
    public Object enable(Integer id){
        ActivityPlan activityPlan = activityPlanService.get(id);
        activityPlan.setEnable(activityPlan.getEnable());
        activityPlanService.update(activityPlan);
        return succeed();
    }

    @ApiOperation(value = "分页查询活动方案")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('activityPlan/queryPage')")
    public Object queryPage(ActivityPlanQueryInfo queryInfo){
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if(sysUser == null){
            return failed("用户信息获取失败");
        }
        Employee employee = employeeDao.get(sysUser.getId());
        if (StringUtils.isEmpty(queryInfo.getOrganId())) {
            queryInfo.setOrganId(employee.getOrganIdList());
        }else if(StringUtils.isEmpty(employee.getOrganIdList())){
            return failed("用户所在分部异常");
        }else {
            List<String> list = Arrays.asList(employee.getOrganIdList().split(","));
            if(!list.containsAll(Arrays.asList(queryInfo.getOrganId().split(",")))){
                return failed("非法请求");
            }
        }
        return succeed(activityPlanService.queryPage(queryInfo));
    }

    @ApiOperation(value = "删除活动方案")
    @PostMapping("/delete")
    @PreAuthorize("@pcs.hasPermissions('activityPlan/delete')")
    public Object delete(Integer id){
        if(Objects.isNull(id)){
            throw new BizException("请指定活动编号!");
        }
        activityPlanService.deleteActivityPlan(id);
        return succeed();
    }
}
