package com.ym.mec.web.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.EduOrganStudentDataDto;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.enums.IndexErrorType;
import com.ym.mec.biz.dal.page.IndexDataQueryInfo;
import com.ym.mec.biz.dal.page.OrganCloudStudyStudentDataQueryInfo;
import com.ym.mec.biz.service.EmployeeService;
import com.ym.mec.biz.service.IndexBaseMonthDataService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Api(tags = "首页")
@RequestMapping()
@RestController
public class IndexController extends BaseController {

	@Autowired
	private MusicGroupDao musicGroupDao;
	
	@Autowired
	private TeacherDao teacherDao;
	
	@Autowired
	private StudentRegistrationDao studentRegistrationDao;
	
	@Autowired
	private VipGroupDao vipGroupDao;
	
	@Autowired
	private DemoGroupDao demoGroupDao;

	@Autowired
	private SysUserFeignService sysUserFeignService;

	@Autowired
	private EmployeeDao employeeDao;

	@Autowired
	private EmployeeService employeeService;
	
	@Autowired
	private SysUserCashAccountDetailDao sysUserCashAccountDetailDao;
	
	@Autowired
	private StudentPaymentOrderDao studentPaymentOrderDao;

	@Autowired
	private IndexBaseMonthDataService indexService;

	@ApiOperation(value = "获取首页数据")
	@GetMapping("/index")
	public Object index() {

		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null || sysUser.getId() == null) {
			return failed(HttpStatus.FORBIDDEN, "请登录");
		}
		Integer userId = sysUser.getId();
		Employee employee = employeeDao.get(userId);
		String organId = employee.getOrganIdList();
		if (sysUser.getIsSuperAdmin()) {
			organId = null;
		}else if(StringUtils.isEmpty(organId)){
			return failed(HttpStatus.INTERNAL_SERVER_ERROR, "分部信息错误");
		}

		Map<String, Integer> musicDatas = musicGroupDao.queryOrganMusicGroupNum(organId);
		
		Map<String, Integer> teacherDatas = teacherDao.queryOrganTeacherNum(organId);
		
		Map<String, Integer> studentDatas = studentRegistrationDao.queryStudentNum(organId);
		
		Map<String, Integer> vipDatas = vipGroupDao.queryVipGroupNum(organId);

		Map<String, Integer> demoDatas = demoGroupDao.queryDemoGroupNum(organId);
		
		Integer musicGroupNum = musicGroupDao.queryCurrentMonthCoursesNum(organId);
		
		Integer vipGroupNum = vipGroupDao.queryCurrentMonthCoursesNum(organId);
		
		Integer demoGroupNum = demoGroupDao.queryCurrentMonthCoursesNum(organId);
		
		Map<String, Object> incomeExpendMap = sysUserCashAccountDetailDao.queryStatisticsWithCurrentMonth(organId);
		
		Map<String, Object> incomeMap = studentPaymentOrderDao.queryIncomeStatisticsWithCurrentMonth(organId);

		ModelMap model = new ModelMap();
		model.put("musicDatas", musicDatas);
		model.put("teacherDatas", teacherDatas);
		model.put("studentDatas", studentDatas);
		model.put("vipDatas", vipDatas);
		model.put("demoDatas", demoDatas);
		model.put("musicGroupNum", musicGroupNum);
		model.put("vipGroupNum", vipGroupNum);
		model.put("demoGroupNum", demoGroupNum);
		model.put("incomeExpendMap", incomeExpendMap);
		model.put("incomeMap", incomeMap);
		return succeed(model);
	}

	@GetMapping("/newIndex")
	public HttpResponseResult newIndex(IndexDataQueryInfo indexDataQueryInfo){
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("用户信息获取失败");
		}
		Employee employee = employeeService.get(sysUser.getId());
		if (StringUtils.isBlank(indexDataQueryInfo.getOrganId())) {
			indexDataQueryInfo.setOrganId(employee.getOrganIdList());
		}else if(StringUtils.isEmpty(employee.getOrganIdList())){
			return failed("用户所在分部异常");
		}else {
			List<String> list = Arrays.asList(employee.getOrganIdList().split(","));
			if(!list.containsAll(Arrays.asList(indexDataQueryInfo.getOrganId().split(",")))){
				return failed("非法请求");
			}
		}
		return succeed(indexService.getIndexBaseData(indexDataQueryInfo));
	}

	@GetMapping("/hasIndexErrData")
	public HttpResponseResult hasIndexErrData(String organId){
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("用户信息获取失败");
		}
			Employee employee = employeeService.get(sysUser.getId());
			if (StringUtils.isBlank(organId)) {
				organId = employee.getOrganIdList();
			}else if(StringUtils.isEmpty(employee.getOrganIdList())){
				return failed("用户所在分部异常");
			}else {
				List<String> list = Arrays.asList(employee.getOrganIdList().split(","));
				if(!list.containsAll(Arrays.asList(organId.split(",")))){
					return failed("非法请求");
				}
			}
		return succeed(indexService.hasIndexErrData(organId));
	}

	@GetMapping("/getIndexErrData")
	public HttpResponseResult getIndexErrData(String organId, IndexErrorType errorType){
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("用户信息获取失败");
		}
		Employee employee = employeeService.get(sysUser.getId());
		if (StringUtils.isBlank(organId)) {
			organId = employee.getOrganIdList();
		}else if(StringUtils.isEmpty(employee.getOrganIdList())){
			return failed("用户所在分部异常");
		}else {
			List<String> list = Arrays.asList(employee.getOrganIdList().split(","));
			if(!list.containsAll(Arrays.asList(organId.split(",")))){
				return failed("非法请求");
			}
		}
		return succeed(indexService.getIndexErrData(organId, errorType));
	}

	@GetMapping("/exportIndexErrData")
	public HttpResponseResult exportIndexErrData(String organId, IndexErrorType errorType){
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("用户信息获取失败");
		}
		Employee employee = employeeService.get(sysUser.getId());
		if (StringUtils.isBlank(organId)) {
			organId = employee.getOrganIdList();
		}else if(StringUtils.isEmpty(employee.getOrganIdList())){
			return failed("用户所在分部异常");
		}else {
			List<String> list = Arrays.asList(employee.getOrganIdList().split(","));
			if(!list.containsAll(Arrays.asList(organId.split(",")))){
				return failed("非法请求");
			}
		}
		return succeed(indexService.exportIndexErrData(organId, errorType));
	}

	@GetMapping("/getRemindMatterData")
	public HttpResponseResult getRemindMatterData(String organId){
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("用户信息获取失败");
		}
			Employee employee = employeeService.get(sysUser.getId());
			if (StringUtils.isBlank(organId)) {
				organId = employee.getOrganIdList();
			}else if(StringUtils.isEmpty(employee.getOrganIdList())){
				return failed("用户所在分部异常");
			}else {
				List<String> list = Arrays.asList(employee.getOrganIdList().split(","));
				if(!list.containsAll(Arrays.asList(organId.split(",")))){
					return failed("非法请求");
				}
			}
		return succeed(indexService.getRemindMatterData(organId));
	}

	@ApiOperation("查询分部云教练学员汇总数据")
	@GetMapping("/organStudentOverView")
	public HttpResponseResult<PageInfo<EduOrganStudentDataDto>> organStudentOverView(OrganCloudStudyStudentDataQueryInfo queryInfo){
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("用户信息获取失败");
		}
		Employee employee = employeeService.get(sysUser.getId());
		List<Integer> organIds = Arrays.stream(employee.getOrganIdList().split(",")).map(id -> Integer.valueOf(id)).filter(id -> !OrganizationService.EXCLUDE_ORGAN_IDS.contains(id)).collect(Collectors.toList());
		return succeed(indexService.organStudentOverView(organIds, queryInfo));
	}
}
