package com.ym.mec.web.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.entity.MusicGroupQuestionnaire;
import com.ym.mec.biz.dal.page.MusicGroupPerformanceQueryInfo;
import com.ym.mec.biz.service.MusicGroupQuestionnaireService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.page.QueryInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

@RequestMapping("musicGroupQuestionnaire")
@Api(tags = "乐团满意度调查")
@RestController
public class MusicGroupQuestionnaireController extends BaseController {

    @Autowired
    private MusicGroupQuestionnaireService musicGroupQuestionnaireService;
    @Autowired
    private SysUserFeignService sysUserFeignService;

    @ApiOperation(value = "新增")
    @PostMapping("/add")
    @PreAuthorize("@pcs.hasPermissions('musicGroupQuestionnaire/add')")
    public Object add(@RequestBody MusicGroupQuestionnaire musicGroupQuestionnaire){
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        if(musicGroupQuestionnaire.getOperatorId() == null){
            musicGroupQuestionnaire.setOperatorId(sysUser.getId());
        }
        musicGroupQuestionnaireService.add(musicGroupQuestionnaire);
        return succeed();
    }

    @ApiOperation(value = "获取题目详情详情")
    @GetMapping("/getQuestionDetail")
    @PreAuthorize("@pcs.hasPermissions('musicGroupQuestionnaire/getQuestionDetail')")
    public Object getDetail() {
        return succeed(musicGroupQuestionnaireService.getQuestionDetail());
    }

    @ApiOperation(value = "删除")
    @PostMapping("/del")
    @PreAuthorize("@pcs.hasPermissions('musicGroupQuestionnaire/del')")
    public Object del(Integer id){
        musicGroupQuestionnaireService.del(id);
        return succeed();
    }

    @ApiOperation(value = "修改")
    @PostMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('musicGroupQuestionnaire/update')")
    public Object update(@RequestBody MusicGroupQuestionnaire musicGroupQuestionnaire){
        musicGroupQuestionnaireService.updateResult(musicGroupQuestionnaire);
        return succeed();
    }

    @ApiOperation(value = "获取详情")
    @PreAuthorize("@pcs.hasPermissions('musicGroupQuestionnaire/get')")
    @GetMapping("/get")
    public Object get(Integer id){
        return succeed(musicGroupQuestionnaireService.getDetail(id));
    }

    @ApiOperation(value = "分页查询")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('musicGroupQuestionnaire/queryPage')")
    public Object queryPage(MusicGroupPerformanceQueryInfo queryInfo){
        return succeed(musicGroupQuestionnaireService.queryPage(queryInfo));
    }
}
