package com.ym.mec.web.controller;


import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.entity.SellOrder;
import com.ym.mec.biz.dal.enums.SellStatus;
import com.ym.mec.biz.dal.enums.SellTypeEnum;
import com.ym.mec.biz.dal.page.RepairStudentQueryInfo;
import com.ym.mec.biz.dal.page.SellOrderQueryInfo;
import com.ym.mec.biz.service.SellOrderService;
import com.ym.mec.biz.service.StudentRepairService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.snaker.engine.access.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@RequestMapping("sellOrder")
@Api(tags = "销售列表")
@RestController
public class SellOrderController extends BaseController {

    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private SellOrderService sellOrderService;
    @Autowired
    private EmployeeDao employeeDao;

    @ApiOperation("销售列表")
    @GetMapping(value = "/queryPage")
    @PreAuthorize("@pcs.hasPermissions('sellOrder/queryPage')")
    public HttpResponseResult<PageInfo<SellOrder>> queryPage(SellOrderQueryInfo queryInfo) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
            Employee employee = employeeDao.get(sysUser.getId());
            if (StringUtils.isEmpty(queryInfo.getOrganIdList())) {
                queryInfo.setOrganIdList(employee.getOrganIdList());
            } else if (StringUtils.isEmpty(employee.getOrganIdList())) {
                return failed("用户所在分部异常");
            } else {
                List<String> list = Arrays.asList(employee.getOrganIdList().split(","));
                if (!list.containsAll(Arrays.asList(queryInfo.getOrganIdList().split(",")))) {
                    return failed("非法请求");
                }
            }
        if (queryInfo.getEndTime() != null) {
            queryInfo.setEndTime(DateUtil.getLastTimeWithDay(queryInfo.getEndTime()));
        }
        return succeed(sellOrderService.queryPage(queryInfo));
    }

    @ApiOperation("更新")
    @PostMapping(value = "/update")
    @PreAuthorize("@pcs.hasPermissions('sellOrder/update')")
    public HttpResponseResult<SellOrder> update(SellOrder sellOrder) {
        SellOrder oldSellOrder = sellOrderService.get(sellOrder.getId());
        oldSellOrder.setEduTeacherId(sellOrder.getEduTeacherId());
        oldSellOrder.setSellCost(sellOrder.getSellCost());
        sellOrderService.update(oldSellOrder);
        return succeed(sellOrder);
    }

    @ApiOperation("退货")
    @PostMapping(value = "/refund")
    @PreAuthorize("@pcs.hasPermissions('sellOrder/refund')")
    public HttpResponseResult<List<SellOrder>> refund(Integer id, Boolean reTry) {
        SellOrder sellOrder = sellOrderService.get(id);

        if (!sellOrder.getStatus().equals(SellStatus.NORMAL)) {
            return failed("当前状态不能退货，请核查");
        }
        if (sellOrder.getType().equals(SellTypeEnum.SCHOOL_BUY)) {
            return failed("学校采购不能退货");
        }
        if (sellOrder.getParentGoodsId() != null && (reTry == null || !reTry)) {
            return failed(HttpStatus.CONTINUE, "组合商品,相关商品将一起退货");
        }
        List<SellOrder> sellOrders = new ArrayList<>();
        if (sellOrder.getParentGoodsId() != null) {
            sellOrders = sellOrderService.getSellOrderByParentGoodsId(sellOrder.getOrderId(), sellOrder.getParentGoodsId());
        } else {
            sellOrders.add(sellOrder);
        }
        return succeed(sellOrderService.refund(sellOrders,true));
    }

}
