package com.ym.mec.web.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.dto.WithdrawDto;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.enums.DealStatusEnum;
import com.ym.mec.biz.dal.enums.TransTypeEnum;
import com.ym.mec.biz.service.StudentWithdrawService;
import com.ym.mec.common.controller.BaseController;

import java.util.Arrays;
import java.util.List;

@RequestMapping("studentWithdraw")
@Api(tags = "提现服务")
@RestController
public class StudentWithdrawController extends BaseController {

	@Autowired
	private StudentWithdrawService studentWithdrawService;

	@Autowired
	private SysUserFeignService sysUserFeignService;

	@Autowired
	private EmployeeDao employeeDao;

	@PostMapping("confirm")
	@ApiOperation(value = "确认提现")
	@PreAuthorize("@pcs.hasPermissions('studentWithdraw/confirm')")
	public Object confirmWithdraw(Integer id, DealStatusEnum status, String memo, String transNo, TransTypeEnum transType, String platformAccountNo) {
		studentWithdrawService.confirmWithdraw(id, status, memo, transNo, transType, platformAccountNo);
		return succeed();
	}

	@ApiOperation(value = "分页查询")
	@GetMapping("/queryPage")
	@PreAuthorize("@pcs.hasPermissions('studentWithdraw/queryPage')")
	public Object queryPage(WithdrawDto queryInfo) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("用户信息获取失败");
		}
			Employee employee = employeeDao.get(sysUser.getId());
			if (StringUtils.isEmpty(queryInfo.getOrganId())) {
				queryInfo.setOrganId(employee.getOrganIdList());
			}else if(StringUtils.isEmpty(employee.getOrganIdList())){
				return failed("用户所在分部异常");
			}else {
				List<String> list = Arrays.asList(employee.getOrganIdList().split(","));
				if(!list.containsAll(Arrays.asList(queryInfo.getOrganId().split(",")))){
					return failed("非法请求");
				}
			}
		return succeed(studentWithdrawService.queryPage(queryInfo));
	}
}
