package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.dao.SysCouponDao;
import com.ym.mec.biz.dal.entity.SysCoupon;
import com.ym.mec.biz.dal.page.SysCouponQueryInfo;
import com.ym.mec.biz.service.SysCouponService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Objects;

@RequestMapping("sysCoupon")
@Api(tags = "优惠券")
@RestController
public class SysCouponController extends BaseController {

	@Autowired
	private SysCouponService sysCouponService;
	@Autowired
	private SysCouponDao sysCouponDao;

	@ApiOperation("新增")
	@PostMapping(value = "add")
	@PreAuthorize("@pcs.hasPermissions('sysCoupon/add')")
	public HttpResponseResult add(SysCoupon sysCoupon) {
		sysCoupon.setStatus(0);
		sysCoupon.setConsumeNum(0);
		sysCoupon.setWarningStatus(0);
		if(Objects.isNull(sysCoupon.getType())){
			return failed("请指定优惠券类型");
		}
		switch (sysCoupon.getType()){
			case DISCOUNT:
				if(Objects.isNull(sysCoupon.getFaceValue())){
					return failed("请指定折扣比例");
				}
				break;
			case FULL_REDUCTION:
				if(Objects.isNull(sysCoupon.getFaceValue())){
					return failed("请指定优惠金额");
				}
				if(Objects.isNull(sysCoupon.getFullAmount())){
					return failed("请指定达标金额");
				}
				break;
			default:
				return failed("请指定优惠券类型");
		}
		switch (sysCoupon.getEffectiveType()){
			case DAYS:
				if(Objects.isNull(sysCoupon.getDeadline())){
					return failed("请指定有效天数");
				}
				break;
			case TIME_BUCKET:
				if(Objects.isNull(sysCoupon.getEffectiveStartTime())||Objects.isNull(sysCoupon.getEffectiveExpireTime())){
					return failed("请指定有效时间段");
				}
				break;
			default:
				return failed("请指定有效期类型");
		}
		SysCoupon sysCouponDaoWithName = sysCouponDao.getWithName(sysCoupon.getName());
		if(Objects.nonNull(sysCouponDaoWithName)){
			return failed("优惠券名称重复");
		}
		return succeed(sysCouponService.insert(sysCoupon));
	}

	@ApiOperation("修改")
	@PostMapping(value = "updateCoupon")
	@PreAuthorize("@pcs.hasPermissions('sysCoupon/updateCoupon')")
	public HttpResponseResult updateCoupon(SysCoupon sysCoupon) {
		sysCouponService.updateCoupon(sysCoupon);
		return succeed();
	}

	@ApiOperation("删除")
	@PostMapping(value = "delete")
	@PreAuthorize("@pcs.hasPermissions('sysCoupon/delete')")
	public HttpResponseResult del(Integer id) {
		SysCoupon sysCoupon = sysCouponService.get(id);
		if(Objects.isNull(sysCoupon)){
			return failed("优惠券信息不存在");
		}
		if(sysCoupon.getConsumeNum()>0){
			return failed("此优惠券已消耗，暂不可删除");
		}
		return succeed(sysCouponService.delete(id));
	}

	@ApiOperation("分页查询")
	@RequestMapping(value = "queryPage")
	@PreAuthorize("@pcs.hasPermissions('sysCoupon/queryPage')")
	public HttpResponseResult queryPage(SysCouponQueryInfo queryInfo) {
		return succeed(sysCouponService.queryPage(queryInfo));
	}
}
