package com.ym.mec.web.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.entity.SysUserCashAccountLog;
import com.ym.mec.biz.dal.page.AccountLogQueryInfo;
import com.ym.mec.biz.service.SysUserCashAccountLogService;
import com.ym.mec.common.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;

@RequestMapping("userCashAccountLog")
@Api(tags = "用户账户退费记录")
@RestController
public class SysUserCashAccountLogController extends BaseController {

	@Autowired
	private SysUserCashAccountLogService sysUserCashAccountLogService;
	@Autowired
	private SysUserFeignService sysUserFeignService;
	@Autowired
	private EmployeeDao employeeDao;

	@ApiOperation(value = "获取退费详情")
	@GetMapping("/get")
	@PreAuthorize("@pcs.hasPermissions('userCashAccountLog/get')")
	public Object get(Long id) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("请重新登录");
		}
		return succeed(sysUserCashAccountLogService.get(id));
	}

	@ApiOperation(value = "修改")
	@GetMapping("/update")
	@PreAuthorize("@pcs.hasPermissions('userCashAccountLog/update')")
	public Object update(SysUserCashAccountLog sysUserCashAccountLog) {
		return succeed(sysUserCashAccountLogService.update(sysUserCashAccountLog));
	}

	@ApiOperation(value = "分页查询收费类型列表")
	@GetMapping("/queryPage")
	@PreAuthorize("@pcs.hasPermissions('userCashAccountLog/queryPage')")
	public Object queryPage(AccountLogQueryInfo queryInfo) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			return failed("用户信息获取失败");
		}
		Employee employee = employeeDao.get(sysUser.getId());
		if (StringUtils.isEmpty(queryInfo.getOrganId())) {
			queryInfo.setOrganId(employee.getOrganIdList());
		}else if(StringUtils.isEmpty(employee.getOrganIdList())){
			return failed("用户所在分部异常");
		}else {
			List<String> list = Arrays.asList(employee.getOrganIdList().split(","));
			if(!list.containsAll(Arrays.asList(queryInfo.getOrganId().split(",")))){
				return failed("非法请求");
			}
		}
		return succeed(sysUserCashAccountLogService.queryPage(queryInfo));
	}
}
