package com.ym.mec.web.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.entity.TeacherSalaryComplaints;
import com.ym.mec.biz.dal.page.TeacherSalaryModifyQueryInfo;
import com.ym.mec.biz.service.TeacherSalaryComplaintsService;
import com.ym.mec.common.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;

@RequestMapping("teacherSalaryComplaints")
@Api(tags = "老师课酬申诉")
@RestController
public class TeacherSalaryComplaintsController extends BaseController {

    @Autowired
    private TeacherSalaryComplaintsService teacherSalaryComplaintsService;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private EmployeeDao employeeDao;

    @ApiOperation(value = "分页获取")
    @RequestMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('teacherSalaryComplaints/queryPage')")
    public Object queryPage(TeacherSalaryModifyQueryInfo queryInfo){
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        Employee employee = employeeDao.get(sysUser.getId());
        if (StringUtils.isEmpty(queryInfo.getOrganId())) {
            queryInfo.setOrganId(employee.getOrganIdList());
        }else if(StringUtils.isEmpty(employee.getOrganIdList())){
            return failed("用户所在分部异常");
        }else {
            List<String> list = Arrays.asList(employee.getOrganIdList().split(","));
            if(!list.containsAll(Arrays.asList(queryInfo.getOrganId().split(",")))){
                return failed("非法请求");
            }
        }
        return succeed(teacherSalaryComplaintsService.queryPage(queryInfo));
    }

    @ApiOperation(value = "获取")
    @RequestMapping("/get")
    @PreAuthorize("@pcs.hasPermissions('teacherSalaryComplaints/get')")
    public Object get(Long id){
        return succeed(teacherSalaryComplaintsService.get(id));
    }

    @ApiOperation(value = "投诉处理")
    @RequestMapping("/complaintsDispose")
    @PreAuthorize("@pcs.hasPermissions('teacherSalaryComplaints/complaintsDispose')")
    public Object complaintsDispose(TeacherSalaryComplaints complaints){
        teacherSalaryComplaintsService.complaintsDispose(complaints);
        return succeed();
    }
}
