package com.ym.mec.web.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.page.CourseHomeWorkTemplateQueryInfo;
import com.ym.mec.biz.dal.page.CourseHomeworkQueryInfo;
import com.ym.mec.biz.dal.page.StudentCourseHomeWorkQueryInfo;
import com.ym.mec.biz.service.CourseHomeworkService;
import com.ym.mec.biz.service.StudentCourseHomeworkService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * @Author zouxuan
 * @Date 2020/11/17
 */
@RequestMapping("webCurseHomework")
@Api(tags = "课后作业")
@RestController
public class WebCourseHomeworkController extends BaseController {

    @Autowired
    private CourseHomeworkService courseHomeworkService;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private EmployeeDao employeeDao;
    @Autowired
    private StudentCourseHomeworkService studentCourseHomeworkService;

    @ApiOperation(value = "分页查询作业列表")
    @GetMapping(value = "/queryHomePage")
    public Object queryPage(CourseHomeWorkTemplateQueryInfo queryInfo){
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        Employee employee = employeeDao.get(sysUser.getId());
        if (StringUtils.isEmpty(queryInfo.getOrganId())) {
            queryInfo.setOrganId(employee.getOrganIdList());
        } else if (StringUtils.isEmpty(employee.getOrganIdList())) {
            return failed("用户所在分部异常");
        } else {
            List<String> list = Arrays.asList(employee.getOrganIdList().split(","));
            if (!list.containsAll(Arrays.asList(queryInfo.getOrganId().split(",")))) {
                return failed("非法请求");
            }
        }
        return succeed(courseHomeworkService.queryHomePage(queryInfo));
    }


    @ApiOperation(value = "获取学生作业记录")
    @GetMapping(value = "/getStudentHomeWorks")
    @PreAuthorize("@pcs.hasPermissions('webCurseHomework/getStudentHomeWorks')")
    public HttpResponseResult getStudentHomeWorks(StudentCourseHomeWorkQueryInfo queryInfo){
        if (queryInfo.getStartTime() != null) {
            queryInfo.setStartTime(DateUtil.trunc(queryInfo.getStartTime()));
        }
        if (queryInfo.getEndTime() != null) {
            queryInfo.setEndTime(DateUtil.getLastTimeWithDay(queryInfo.getEndTime()));
        }
        return succeed(studentCourseHomeworkService.getStudentHomeWorks(queryInfo));
    }

}
