package com.ym.mec.web.controller.education;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dto.EduOrganStudentDataDto;
import com.ym.mec.biz.dal.dto.StatDto;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.page.StudentQueryInfo;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.biz.service.StudentService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author Joburgess
 * @Date 2021/8/17 0017
 */
@RequestMapping("eduStudent")
@Api(tags = "学生信息服务")
@RestController
public class EduStudentController extends BaseController {

    @Autowired
    private StudentService studentService;
    @Autowired
    private OrganizationService organizationService;
	@Autowired
	private SysUserFeignService sysUserFeignService;

    @ApiOperation(value = "云教练学员数据")
    @GetMapping("/organStudentData")
    public HttpResponseResult<StatDto> organStudentData(StudentQueryInfo queryInfo){
        return succeed(studentService.organStudentData(queryInfo));
    }

    @ApiOperation(value = "分部云教练学员数据预览")
    @GetMapping("/organStudentOverView")
    public HttpResponseResult<List<EduOrganStudentDataDto>> organStudentOverView() throws Exception {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if(sysUser == null){
			throw new BizException("获取用户信息失败");
		}
		Integer userId = sysUser.getId();
        List<Organization> organizations = organizationService.queryEmployeeOrgan(userId,null);
        if(CollectionUtils.isEmpty(organizations)){
            return succeed(Collections.emptyList());
        }
        List<Integer> organIds = organizations.stream().map(Organization::getId).collect(Collectors.toList());
        return succeed(studentService.organStudentOverView(organIds));
    }

}
